@@ -18,11 +18,10 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 """
 
-import syslog
-
-from util import m18nc, m18ncE, logWarning, logException, logMessage
+from util import m18nc, m18ncE, logWarning, logException, logDebug
 from sound import Voice, Sound
-from scoringengine import Meld
+from meld import Meld
+from common import InternalParameters, Debug
 
 class Message(object):
     """those are the message types between client and server. They have no state
@@ -34,6 +33,7 @@ class Message(object):
         """those are the english values"""
         self.name = name or self.__class__.__name__.replace('Message', '')
         self.i18nName = self.name
+        self.shortcut = shortcut
         # do not use a numerical value because that could easier
         # change with software updates
         Message.defined[self.name] = self
@@ -46,6 +46,8 @@ class Message(object):
 
 class MessageFromServer(Message):
     """those classes are used for messages from server to client"""
+    # if sendScore is True, this message will send info about player scoring, so the clients can compare
+    sendScore = False
     def __init__(self, name=None):
         Message.__init__(self, name)
 
@@ -57,7 +59,6 @@ class MessageFromClient(Message):
     """those classes are used for messages from client to server"""
     def __init__(self, name=None, shortcut=None):
         Message.__init__(self, name, shortcut)
-        self.shortcut = shortcut
         self.i18nName = m18nc('kajongg', self.name)
         self.notifyAtOnce = False
 
@@ -103,10 +104,7 @@ class MessageKong(NotifyAtOnceMessage):
             shortcut=m18ncE('kajongg game dialog:Key for Kong', 'K'))
     def serverAction(self, table, msg):
         """the server mirrors that and tells all others"""
-        if msg.player == table.game.activePlayer:
-            table.declareKong(msg.player, msg.args[0])
-        else:
-            table.claimTile(msg.player, self, msg.args[0], Message.CalledKong)
+        table.claimTile(msg.player, self, msg.args[0], Message.CalledKong)
 
 class MessageChow(NotifyAtOnceMessage):
     """the client said chow"""
@@ -126,7 +124,7 @@ class MessageBonus(MessageFromClient):
     def serverAction(self, table, msg):
         """the server mirrors that"""
         if self.isActivePlayer(table, msg):
-            table.pickedBonus(msg.player, msg.args[0])
+            table.pickTile()
 
 class MessageMahJongg(NotifyAtOnceMessage):
     """the client says mah jongg"""
@@ -171,19 +169,28 @@ class MessageDiscard(MessageFromClient):
         """the server mirrors that action"""
         table.discard(msg)
 
+class MessageProposeGameId(MessageFromServer):
+    """the game server proposes a new game id. We check if it is available
+    in our local data base - we want to use the same gameid everywhere"""
+    def clientAction(self, client, move):
+        """ask the client"""
+        # move.source are the players in seating order
+        # we cannot just use table.playerNames - the seating order is now different (random)
+        client.reserveGameId(move.gameid)
+
 class MessageReadyForGameStart(MessageFromServer):
     """the game server asks us if we are ready for game start"""
     def clientAction(self, client, move):
         """ask the client"""
         # move.source are the players in seating order
         # we cannot just use table.playerNames - the seating order is now different (random)
-        client.readyForGameStart(move.tableid, move.seed, move.source, shouldSave=move.shouldSave)
+        client.readyForGameStart(move.tableid, move.gameid, move.seed, move.source, shouldSave=move.shouldSave)
 
 class MessageReadyForHandStart(MessageFromServer):
     """the game server asks us if we are ready for a new hand"""
     def clientAction(self, client, move):
         """ask the client"""
-        client.readyForHandStart(move.source, move.rotate)
+        client.readyForHandStart(move.source, move.rotateWinds)
 
 class MessageInitHand(MessageFromServer):
     """the game server tells us to prepare a new hand"""
@@ -193,10 +200,16 @@ class MessageInitHand(MessageFromServer):
         client.game.showField()
 
 class MessageSetConcealedTiles(MessageFromServer):
+    """the game server assigns tiles to player"""
+    def clientAction(self, client, move):
+        """set tiles for player"""
+        client.game.setConcealedTiles(move.source)
+
+class MessageShowConcealedTiles(MessageFromServer):
     """the game server assigns tiles to player"""
     def clientAction(self, dummyClient, move):
         """set tiles for player"""
-        move.player.setConcealedTiles(move.source)
+        move.player.showConcealedTiles(move.source, move.show)
 
 class MessageSaveHand(MessageFromServer):
     """the game server tells us to save the hand"""
@@ -212,9 +225,11 @@ class MessagePopupMsg(MessageFromServer):
 
 class MessageHasDiscarded(MessageFromServer):
     """the game server tells us who discarded which tile"""
+ #   sendScore = True
+
     def clientAction(self, client, move):
         """execute the discard locally"""
-        if client.isHumanClient():
+        if client.isHumanClient() and InternalParameters.field:
             move.player.handBoard.setEnabled(False)
         move.player.speak(move.tile)
         if move.tile != move.player.lastTile:
@@ -235,8 +250,8 @@ class MessagePickedTile(MessageFromServer):
     """the game server tells us who picked a tile"""
     def clientAction(self, client, move):
         """mirror the picked tile"""
-        client.game.wall.dealTo(deadEnd=move.deadEnd)
-        client.game.pickedTile(move.player, move.source, move.deadEnd)
+        assert client.game.pickedTile(move.player, move.deadEnd, tileName=move.source).element == move.source,  \
+            (move.player.lastTile, move.source)
         if client.thatWasMe(move.player):
             if move.source[0] in 'fy':
                 client.answers.append((Message.Bonus, move.source))
@@ -267,19 +282,12 @@ class MessageCalledKong(MessageFromServer):
         """mirror kong call"""
         client.called(move)
 
-class MessagePickedBonus(MessageFromServer):
-    """the game server tells us who picked a bonus tile"""
-    def clientAction(self, client, move):
-        """assign the bonus tile to a player"""
-        assert not client.thatWasMe(move.player)
-        move.player.makeTilesKnown(move.source)
-
 class MessageActivePlayer(MessageFromServer):
     """the game server tells us whose turn it is"""
     def clientAction(self, client, move):
         """set the active player"""
         client.game.activePlayer = move.player
-        if client.isHumanClient():
+        if client.isHumanClient() and InternalParameters.field:
             move.player.handBoard.setEnabled(True)
 
 class MessageMadeOriginalCall(MessageFromServer):
@@ -337,9 +345,13 @@ class MessageDeclaredKong(MessageFromServer):
         prompts = None
         client.invalidateOriginalCall(move.player)
         if not client.thatWasMe(move.player):
-            move.player.makeTilesKnown(move.source)
+            if len(move.source) != 4 or move.source[0].istitle():
+                # do not do this when adding a 4th tile to an exposed pung
+                move.player.showConcealedTiles(move.source)
+            else:
+                move.player.showConcealedTiles(move.source[3:4])
             prompts = [Message.NoClaim, Message.MahJongg]
-        move.exposedMeld = move.player.exposeMeld(move.source, claimed=False)
+        move.exposedMeld = move.player.exposeMeld(move.source)
         if prompts:
             client.ask(move, prompts)
 
@@ -347,54 +359,64 @@ class MessageRobbedTheKong(MessageFromServer):
     """the game server tells us who robbed the kong"""
     def clientAction(self, client, move):
         """mirror the action locally"""
-        prevMove = None
-        for oldMove in reversed(client.moves):
-            if oldMove.message == Message.DeclaredKong:
-                prevMove = oldMove
-                break
-        assert prevMove.message == Message.DeclaredKong
+        prevMove = client.game.lastMoves(only=[Message.DeclaredKong]).next()
         prevKong = Meld(prevMove.source)
-        prevMove.player.robTile(prevKong.pairs[0])
+        prevMove.player.robTile(prevKong.pairs[0].capitalize())
         move.player.lastSource = 'k'
 
+class MessagePlayedDangerous(MessageFromServer):
+    """the game server tells us who played dangerous game"""
+    def clientAction(self, client, move):
+        """mirror the dangerous game action locally"""
+        move.player.popupMsg(m18nc('kajongg', 'Dangerous Game'))
+        move.player.playedDangerous = True
+        client.ask(move, [Message.OK])
+
 class MessageHasNoChoice(MessageFromServer):
     """the game server tells us who had no choice avoiding dangerous game"""
     def __init__(self):
         MessageFromServer.__init__(self)
         self.move = None
-        self.prevConc = None
 
     def clientAction(self, client, move):
         """mirror the no choice action locally"""
         self.move = move
         move.player.popupMsg(m18nc('kajongg', 'No Choice'))
         move.player.claimedNoChoice = True
-        self.prevConc = move.player.concealedTiles
-        move.player.setConcealedTiles(move.tile)
+        move.player.showConcealedTiles(move.tile)
         client.ask(move, [Message.OK], self.hideConcealedAgain)
 
-    def hideConcealedAgain(self, dummy):
+    def hideConcealedAgain(self, dummyResult=None):
         """only show them for explaining the 'no choice'"""
-        self.move.player.setConcealedTiles(self.prevConc)
+        self.move.player.showConcealedTiles(self.move.tile, False)
+
+class MessageUsedDangerousFrom(MessageFromServer):
+    """the game server tells us somebody claimed a dangerous tile"""
+    def clientAction(self, client, move):
+        fromPlayer = client.game.playerByName(move.source)
+        move.player.usedDangerousFrom = fromPlayer
+        if Debug.dangerousGame:
+            logDebug('%s claimed a dangerous tile discarded by %s' % \
+                (move.player, fromPlayer))
 
 class MessageDeclaredMahJongg(MessageFromServer):
     """the game server tells us who said mah jongg"""
+    sendScore = True
+
     def clientAction(self, dummyClient, move):
         """mirror the mahjongg action locally. Check if the balances are correct."""
         move.player.declaredMahJongg(move.source, move.withDiscard,
             move.lastTile, move.lastMeld)
-        if move.player.balance != move.winnerBalance:
-            logException('WinnerBalance is different for %s! player:%d, remote:%d,hand:%s' % \
-                (move.player, move.player.balance, move.winnerBalance, move.player.computeHandContent()))
+
+class MessageDraw(MessageFromServer):
+    """the game server tells us nobody said mah jongg"""
+    sendScore = True
 
 class MessageError(MessageFromServer):
     """a client errors"""
-    def clientAction(self, client, move):
+    def clientAction(self, dummyClient, move):
         """show the error message from server"""
-        if client.perspective:
-            logWarning(move.source) # show messagebox
-        else:
-            logMessage(move.source, prio=syslog.LOG_WARNING)
+        logWarning(move.source)
 
 class MessageNO(MessageFromClient):
     """a client says no"""
@@ -427,4 +449,4 @@ def __scanSelf():
                         msg = glob()
                         type.__setattr__(Message, msg.name.replace(' ', ''), msg)
 
-__scanSelf()
+__scanSelf()
\ No newline at end of file
