@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import sys
 import os
-from util import logMessage, logException, m18n, m18nc, isAlive
+from util import logError, m18n, m18nc, isAlive
 import common
 from common import WINDS, LIGHTSOURCES, InternalParameters
 import cgitb, tempfile, webbrowser
@@ -44,22 +44,18 @@ try:
     from PyQt4.QtCore import Qt, QVariant, SIGNAL, \
         QEvent, QMetaObject, PYQT_VERSION_STR, QString
     from PyQt4.QtGui import QPushButton, QMessageBox
-    from PyQt4.QtGui import QWidget
+    from PyQt4.QtGui import QWidget, QColor, QBrush
     from PyQt4.QtGui import QGridLayout
     from PyQt4.QtGui import QDialogButtonBox
-    from PyQt4.QtGui import QComboBox
+    from PyQt4.QtGui import QComboBox, QSlider, QHBoxLayout, QLabel
     from PyQt4.QtGui import QVBoxLayout, QSpacerItem, QSizePolicy, QCheckBox
 except ImportError, e:
     NOTFOUND.append('Package python-qt4: PyQt4: %s' % e)
 
 try:
-    from zope.interface import implements # pylint: disable-msg=W0611
+    from zope.interface import implements # pylint: disable=W0611
 except ImportError, e:
     NOTFOUND.append('Package python-zope-interface missing: %s' % e)
-try:
-    from twisted.python.failure import Failure
-except ImportError, e:
-    NOTFOUND.append('Package twisted missing: %s' % e)
 
 try:
     from PyKDE4.kdeui import KApplication, KStandardAction, KAction, KToggleFullScreenAction
@@ -70,8 +66,8 @@ except ImportError, e :
 try:
     from query import Query
     from tile import Tile
-    from board import WindLabel, FittingView, \
-        HandBoard, SelectorBoard, DiscardBoard, MJScene
+    from board import WindLabel, FittingView, SelectorBoard, DiscardBoard, MJScene
+    from handboard import HandBoard
     from playerlist import PlayerList
     from tileset import Tileset
     from background import Background
@@ -86,15 +82,16 @@ try:
     from backgroundselector import BackgroundSelector
     from sound import Sound
     from uiwall import UIWall
+    from animation import animate, afterCurrentAnimationDo, Animated
 
-    from game import Game, Players, Player
+    from game import Game, ScoringGame, Players, Player
 
 except ImportError, e:
     NOTFOUND.append('kajongg is not correctly installed: modules: %s' % e)
 
 if len(NOTFOUND):
     MSG = "\n".join(" * %s" % s for s in NOTFOUND)
-    logMessage(MSG)
+    logError(MSG)
     os.popen("kdialog --sorry '%s'" % MSG)
     sys.exit(3)
 
@@ -107,10 +104,21 @@ class PlayConfigTab( QWidget):
     def setupUi(self):
         """layout the window"""
         vlayout = QVBoxLayout(self)
-        self.kcfg_showShadows = QCheckBox(m18n('show tile shadows'), self)
+        sliderLayout = QHBoxLayout()
+        self.kcfg_showShadows = QCheckBox(m18n('Show tile shadows'), self)
         self.kcfg_showShadows.setObjectName('kcfg_showShadows')
         self.kcfg_rearrangeMelds = QCheckBox(m18n('Rearrange undisclosed tiles to melds'), self)
         self.kcfg_rearrangeMelds.setObjectName('kcfg_rearrangeMelds')
+        self.kcfg_showOnlyPossibleActions = QCheckBox(m18n('Show only possible actions'))
+        self.kcfg_showOnlyPossibleActions.setObjectName('kcfg_showOnlyPossibleActions')
+        self.kcfg_animationSpeed = QSlider(self)
+        self.kcfg_animationSpeed.setObjectName('kcfg_animationSpeed')
+        self.kcfg_animationSpeed.setOrientation(Qt.Horizontal)
+        self.kcfg_animationSpeed.setSingleStep(1)
+        lblSpeed = QLabel(m18n('Animation speed:'))
+        lblSpeed.setBuddy(self.kcfg_animationSpeed)
+        sliderLayout.addWidget(lblSpeed)
+        sliderLayout.addWidget(self.kcfg_animationSpeed)
         self.kcfg_useSounds = QCheckBox(m18n('Use sounds if available'), self)
         self.kcfg_useSounds.setObjectName('kcfg_useSounds')
         self.kcfg_uploadVoice = QCheckBox(m18n('Let others hear my voice'), self)
@@ -121,8 +129,10 @@ class PlayConfigTab( QWidget):
         spacerItem = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Expanding)
         vlayout.addWidget(self.kcfg_showShadows)
         vlayout.addWidget(self.kcfg_rearrangeMelds)
+        vlayout.addWidget(self.kcfg_showOnlyPossibleActions)
         vlayout.addWidget(self.kcfg_useSounds)
         vlayout.addWidget(self.kcfg_uploadVoice)
+        vlayout.addLayout(sliderLayout)
         vlayout.addItem(spacerItem)
         self.setSizePolicy(pol)
         self.retranslateUi()
@@ -163,10 +173,10 @@ class ConfigDialog(KConfigDialog):
 
     def accept(self):
         """commit transaction"""
+        Query.dbhandle.commit() # commit renames and deletes of rulesets
         if self.rulesetSelector.save():
-            if Query.dbhandle.commit():
-                KConfigDialog.accept(self)
-                return
+            KConfigDialog.accept(self)
+            return
         KMessageBox.sorry(None, m18n('Cannot save your ruleset changes.<br>' \
             'You probably introduced a duplicate name. <br><br >Message from database:<br><br>' \
            '<message>%1</message>', Query.lastError))
@@ -205,31 +215,32 @@ class SelectPlayers(SelectRuleset):
         self.setWindowTitle(m18n('Select four players') + ' - Kajongg')
         self.names = None
         self.nameWidgets = []
+        self.allNames = list(x[0] for x in Query('select name, id from player where'
+                ' not name like "ROBOT %" and not exists(select 1 from'
+                ' server where server.lastname=player.name)').records)
         for idx, wind in enumerate(WINDS):
             cbName = QComboBox()
             # increase width, we want to see the full window title
             cbName.setMinimumWidth(350) # is this good for all platforms?
             # add all player names belonging to no host
-            cbName.addItems(list(x[1] for x in Players.allNames.values() if x[0] == ''))
+            cbName.addItems(self.allNames)
             self.grid.addWidget(cbName, idx+1, 1)
             self.nameWidgets.append(cbName)
             self.grid.addWidget(WindLabel(wind), idx+1, 0)
             self.connect(cbName, SIGNAL('currentIndexChanged(int)'),
                 self.slotValidate)
 
-        query = Query("select p0,p1,p2,p3 from game where server='' and game.id = (select max(id) from game)")
+        query = Query("select p0,p1,p2,p3 from game where seed is null and game.id = (select max(id) from game)")
         if len(query.records):
-            for pidx in range(4):
-                playerId = query.records[0][pidx]
+            for pidx, playerId in enumerate(query.records[0]):
                 try:
-                    (host, playerName)  = Players.allNames[playerId]
-                    assert host == ''
+                    playerName  = Players.allNames[playerId]
                     cbName = self.nameWidgets[pidx]
                     playerIdx = cbName.findText(playerName)
                     if playerIdx >= 0:
                         cbName.setCurrentIndex(playerIdx)
                 except KeyError:
-                    logMessage('database is inconsistent: player with id %d is in game but not in player' \
+                    logError('database is inconsistent: player with id %d is in game but not in player' \
                                % playerId)
         self.slotValidate()
 
@@ -242,18 +253,18 @@ class SelectPlayers(SelectRuleset):
         changedCombo = self.sender()
         if not isinstance(changedCombo, QComboBox):
             changedCombo = self.nameWidgets[0]
-        usedNames = set([str(x.currentText()) for x in self.nameWidgets])
-        allNames = set(x[1] for x in Players.allNames.values() if x[0] == '')
+        usedNames = set([unicode(x.currentText()) for x in self.nameWidgets])
+        allNames = set(self.allNames)
         unusedNames = allNames - usedNames
-        foundNames = [str(changedCombo.currentText())]
+        foundNames = [unicode(changedCombo.currentText())]
         for combo in self.nameWidgets:
             if combo is not changedCombo:
-                if str(combo.currentText()) in foundNames:
+                if unicode(combo.currentText()) in foundNames:
                     if not unusedNames:
                         break
                     combo.setItemText(combo.currentIndex(), unusedNames.pop())
-                foundNames.append(str(combo.currentText()))
-        self.names = list(str(cbName.currentText()) for cbName in self.nameWidgets)
+                foundNames.append(unicode(combo.currentText()))
+        self.names = list(unicode(cbName.currentText()) for cbName in self.nameWidgets)
         valid = len(set(self.names)) == 4
         self.buttonBox.button(QDialogButtonBox.Ok).setEnabled(valid)
 
@@ -261,33 +272,17 @@ class VisiblePlayer(Player):
     """this player instance has a visual representation"""
     def __init__(self, game, idx):
         assert game
+        self.handBoard = None # because Player.init calls clearHand()
         Player.__init__(self, game)
         self.idx = idx
         self.front = game.wall[idx]
         self.manualRuleBoxes = []
         self.handBoard = HandBoard(self)
         self.handBoard.setVisible(False)
-        if common.PREF.showShadows:
-            self.handBoard.setPos(yHeight= 1.5)
-        else:
-            self.handBoard.setPos(yHeight= 1.0)
-
-    def removeTile(self, tileName):
-        """player loses tile"""
-        Player.removeTile(self, tileName)
-        self.syncHandBoard()
-
-    def exposeMeld(self, meldTiles, claimed=True):
-        """player exposes meld"""
-        result = Player.exposeMeld(self, meldTiles, claimed)
-        self.syncHandBoard()
-        return result
 
     def clearHand(self):
         """clears attributes related to current hand"""
         self.manualRuleBoxes = []
-        if self.handBoard:
-            self.handBoard.clear()
         Player.clearHand(self)
 
     def hasManualScore(self):
@@ -296,37 +291,25 @@ class VisiblePlayer(Player):
             return InternalParameters.field.scoringDialog.spValues[self.idx].isEnabled()
         return False
 
-    def syncHandBoard(self, tileName=None):
+    def syncHandBoard(self, adding=None):
         """update display of handBoard. Set Focus to tileName."""
-        myBoard = self.handBoard
-        myBoard.clear()
-        for meld in self.exposedMelds:
-            myBoard.receive(meld.pairs, None, False)
-        for tile in self.bonusTiles:
-            myBoard.receive(tile, None, False)
-        if self.concealedMelds:
-            # hand has ended
-            for meld in self.concealedMelds:
-                myBoard.receive(meld.pairs, None, True)
-        else:
-            tileStr = ''.join(self.concealedTiles)
-            content = HandContent.cached(self.game.ruleset, tileStr)
-            for meld in content.sortedMelds.split():
-                myBoard.receive(meld, None, True)
-            for exposed in myBoard.exposedTiles():
-                exposed.focusable = False
-            tiles = myBoard.lowerHalfTiles()
-            if tiles:
-                if self == self.game.myself and tileName and tileName[0] not in 'fy':
-                    myBoard.focusTile = [x for x in tiles if x.element == tileName][-1]
-                elif tiles[-1].element != 'Xy' and tiles[-1].focusable:
-                    myBoard.focusTile = tiles[-1]
-            InternalParameters.field.centralView.scene().setFocusItem(myBoard.focusTile)
+        self.handBoard.sync(adding)
 
-    def robTile(self, tileName):
-        """used for robbing the kong"""
-        Player.robTile(self, tileName)
-        self.syncHandBoard()
+    def moveMeld(self, meld):
+        """a meld moves within our handBoard"""
+        assert meld.tiles[0].board == self.handBoard
+        self.removeMeld(meld)
+        self.addMeld(meld)
+
+    def colorizeName(self):
+        """set the color to be used for showing the player name on the wall"""
+        if self == self.game.activePlayer and self.game.client:
+            color = Qt.blue
+        elif InternalParameters.field.tilesetName == 'jade':
+            color = Qt.white
+        else:
+            color = Qt.black
+        self.front.nameLabel.setBrush(QBrush(QColor(color)))
 
     def refreshManualRules(self, sender=None):
         """update status of manual rules"""
@@ -358,7 +341,7 @@ class VisiblePlayer(Player):
 
     def __mjstring(self, singleRule):
         """compile hand info into a string as needed by the scoring engine"""
-        winds = self.wind.lower() + 'eswn'[self.game.roundsFinished]
+        winds = self.wind.lower() + 'eswn'[self.game.roundsFinished % 4]
         wonChar = 'm'
         if self == self.game.winner:
             wonChar = 'M'
@@ -396,7 +379,7 @@ class VisiblePlayer(Player):
             return Player.computeHandContent(self, withTile=withTile, robbedTile=robbedTile)
         if not self.handBoard:
             return None
-        string = ' '.join([self.handBoard.scoringString(), self.__mjstring(singleRule), self.__lastString()])
+        string = ' '.join([self.scoringString(), self.__mjstring(singleRule), self.__lastString()])
         if game.eastMJCount == 8 and self == game.winner and self.wind == 'E':
             cRules = [game.ruleset.findRule('XEAST9X')]
         else:
@@ -417,30 +400,19 @@ class VisiblePlayer(Player):
         if isAlive(self.front.message):
             self.front.message.setVisible(False)
 
-def quit2():
-    """2nd stage: twisted reactor is already stopped"""
-    StateSaver.saveAll()
-    InternalParameters.app.quit()
-#       sys.exit(0)
-    # pylint: disable-msg=W0212
-    os._exit(0) # TODO: should be sys.exit but that hangs since updating
-    # from karmic 32 bit to lucid 64 bit. os._exit does not clean up or flush buffers
-
-
 class PlayField(KXmlGuiWindow):
     """the main window"""
-    # pylint: disable-msg=R0902
-    # pylint: we need more than 10 instance attributes
+    # pylint: disable=R0902
+    # pylint we need more than 10 instance attributes
 
-    def __init__(self, reactor):
+    def __init__(self):
         # see http://lists.kde.org/?l=kde-games-devel&m=120071267328984&w=2
-        self.reactor = reactor
         InternalParameters.field = self
         self.game = None
         self.ignoreResizing = 1
         super(PlayField, self).__init__()
         self.background = None
-        self.settingsChanged = False
+        self.showShadows = None
         self.clientDialog = None
 
         self.playerWindow = None
@@ -453,8 +425,18 @@ class PlayField(KXmlGuiWindow):
         self.applySettings()
         self.setupGUI()
         self.retranslateUi()
-        if InternalParameters.autoPlay:
-            self.playGame()
+
+    def sizeHint(self):
+        """give the main window a sensible default size"""
+        result = KXmlGuiWindow.sizeHint(self)
+        result.setWidth(result.height() * 3 / 2) # we want space to the right for the buttons
+        # the default is too small. Use at least 2/3 of screen height and 1/2 of screen width:
+        available = KApplication.kApplication().desktop().availableGeometry()
+        height = max(result.height(), available.height() * 2 / 3)
+        width = max(result.width(), available.width() / 2)
+        result.setHeight(height)
+        result.setWidth(width)
+        return result
 
     def resizeEvent(self, event):
         """Use this hook to determine if we want to ignore one more resize
@@ -484,7 +466,6 @@ class PlayField(KXmlGuiWindow):
         if self.scoringDialog:
             self.scoringDialog.slotInputChanged()
         if self.game and not self.game.finished():
-            self.game.checkSelectorTiles()
             self.game.wall.decoratePlayer(handBoard.player)
         # first decorate walls - that will compute player.handBoard for explainView
         if self.explainView:
@@ -511,12 +492,6 @@ class PlayField(KXmlGuiWindow):
         self.connect(res, SIGNAL('toggled(bool)'), self.__toggleWidget)
         return res
 
-    def tileClicked(self, event, tile):
-        """save the clicked tile, we need it when dropping things into boards"""
-        self.centralScene.clickedTile = tile
-        self.centralScene.clickedTileEvent = event
-        self.selectorBoard.setAcceptDrops(tile.board != self.selectorBoard)
-
     def setupUi(self):
         """create all other widgets
         we could make the scene view the central widget but I did
@@ -546,8 +521,6 @@ class PlayField(KXmlGuiWindow):
         self.selectorBoard.setVisible(False)
         scene.addItem(self.selectorBoard)
 
-        self.connect(scene, SIGNAL('tileClicked'), self.tileClicked)
-
         self.setCentralWidget(centralWidget)
         self.centralView.setScene(scene)
         self.centralView.setFocusPolicy(Qt.StrongFocus)
@@ -572,15 +545,19 @@ class PlayField(KXmlGuiWindow):
             Qt.Key_T, actionData=ScoreTable)
         self.actionExplain = self.kajonggToggleAction("explain", "applications-education",
             Qt.Key_E, actionData=ExplainView)
+        self.actionAutoPlay = self.kajonggAction("demoMode", "arrow-right", None, Qt.Key_D)
+        self.actionAutoPlay.setCheckable(True)
+        self.connect(self.actionAutoPlay, SIGNAL('toggled(bool)'), self.__toggleDemoMode)
+        self.actionAutoPlay.setChecked(InternalParameters.autoPlay)
         QMetaObject.connectSlotsByName(self)
 
     def showWall(self):
         """shows the wall according to the game rules (lenght may vary)"""
-        self.game.wall = UIWall(self.game)
+        UIWall(self.game)
         if self.discardBoard:
             # scale it such that it uses the place within the wall optimally.
             # we need to redo this because the wall length can vary between games.
-            self.discardBoard.scale()
+            self.discardBoard.maximize()
 
     def genPlayers(self):
         """generate four default VisiblePlayers"""
@@ -592,84 +569,72 @@ class PlayField(KXmlGuiWindow):
 
     def quit(self):
         """exit the application"""
-        if self.game:
-            return self.abortGame(self.gameClosed)
-        else:
-            self.gameClosed()
-            return True
+        return self.abortGame(HumanClient.gameClosed)
 
-    def gameClosed(self, result=None):
-        """called if we want to quit, after the game has been closed"""
-        if isinstance(result, Failure):
-            logException(result)
-        self.reactor.stop()
-        HumanClient.stopLocalServers()
-        # we may be in a Deferred callback generated in abortGame which would
-        # catch sys.exit as an exception
-        # and the qt4reactor does not quit the app when being stopped
-        self.connect(self, SIGNAL('reactorStopped'), quit2)
-        self.emit(SIGNAL('reactorStopped'))
+    def abortGame(self, callback=None):
+        """if a game is active, abort it"""
+        self.actionAutoPlay.setChecked(False)
+        if self.game:
+            if self.game.client:
+                return self.game.client.abortGame(callback)
+            else:
+                self.game.close()
+        if callback:
+            callback()
+        return True
 
     def closeEvent(self, event):
         """somebody wants us to close, maybe ALT-F4 or so"""
         if not self.quit():
             event.ignore()
 
-    def __moveTile(self, tile, windIndex, lowerHalf):
+    def __moveTile(self, tile, wind, lowerHalf):
         """the user pressed a wind letter or X for center, wanting to move a tile there"""
-        tile = self.centralScene.focusItem()
         # this tells the receiving board that this is keyboard, not mouse navigation>
         # needed for useful placement of the popup menu
-        self.centralScene.clickedTile = None
-        # check opacity because we might be positioned on a hole
-        if isinstance(tile, Tile) and tile.opacity:
-            currentBoard = tile.board
-            if windIndex == 4:
-                receiver = self.selectorBoard
-                if receiver.isEnabled():
-                    receiver.receive(tile)
-            else:
-                targetWind = WINDS[windIndex]
-                for player in self.game.players:
-                    if player.wind == targetWind:
-                        receiver = player.handBoard
-                        if receiver.isEnabled(lowerHalf):
-                            receiver.receive(tile, self.centralView, lowerHalf=lowerHalf)
-            if receiver.isEnabled() and not currentBoard.allTiles():
-                self.centralView.scene().setFocusItem(receiver.focusTile)
-            else:
-                self.centralView.scene().setFocusItem(currentBoard.focusTile)
+        assert self.game.isScoringGame()
+        assert isinstance(tile, Tile), (tile, str(tile))
+        currentBoard = tile.board
+        tile, meld = currentBoard.dragObject(tile)
+        if wind == 'X':
+            receiver = self.selectorBoard
+        else:
+            receiver = self.game.players[wind].handBoard
+        receiver.dropHere(tile, meld, lowerHalf)
+
+    def __navigateScoringGame(self, event):
+        """keyboard navigation in a scoring game"""
+        mod = event.modifiers()
+        key = event.key()
+        wind = chr(key%128)
+        moveCommands = m18nc('kajongg:keyboard commands for moving tiles to the players ' \
+            'with wind ESWN or to the central tile selector (X)', 'ESWNX')
+        tile = self.centralScene.focusItem().tile
+        if wind in moveCommands:
+            # translate i18n wind key to ESWN:
+            wind = 'ESWNX'[moveCommands.index(wind)]
+            self.__moveTile(tile, wind, mod &Qt.ShiftModifier)
+            return True
+        if key == Qt.Key_Tab and self.game:
+            tabItems = [self.selectorBoard]
+            tabItems.extend(list(p.handBoard for p in self.game.players if p.handBoard.tiles))
+            tabItems.append(tabItems[0])
+            currentBoard = tile.board if isinstance(tile, Tile) else None
+            currIdx = 0
+            while tabItems[currIdx] != currentBoard and currIdx < len(tabItems) -2:
+                currIdx += 1
+            tabItems[currIdx+1].hasFocus = True
+            return True
 
     def keyPressEvent(self, event):
         """navigate in the selectorboard"""
         mod = event.modifiers()
-        if not mod in (Qt.NoModifier, Qt.ShiftModifier):
-            # no other modifier is allowed
-            KXmlGuiWindow.keyPressEvent(self, event)
-            return
-        key = event.key()
-        tile = self.centralScene.focusItem()
-        currentBoard = tile.board if isinstance(tile, Tile) else None
-        wind = chr(key%128)
-        moveCommands = m18nc('kajongg:keyboard commands for moving tiles to the players ' \
-            'with wind ESWN or to the central tile selector (X)', 'ESWNX')
-        if wind in moveCommands:
-            self.__moveTile(tile, moveCommands.index(wind), mod &Qt.ShiftModifier)
-            return
-        if key == Qt.Key_Tab and self.game:
-            tabItems = []
-            if self.selectorBoard.isEnabled():
-                tabItems = [self.selectorBoard]
-            tabItems.extend(list(p.handBoard for p in self.game.players if p.handBoard.focusTile))
-            tabItems.append(tabItems[0])
-            currIdx = 0
-            while tabItems[currIdx] != currentBoard and currIdx < len(tabItems) -2:
-                currIdx += 1
-            newItem = tabItems[currIdx+1].focusTile
-            self.centralView.scene().setFocusItem(newItem)
-            return
-        if self.clientDialog:
-            self.clientDialog.keyPressEvent(event)
+        if mod in (Qt.NoModifier, Qt.ShiftModifier):
+            if self.game and self.game.isScoringGame():
+                if self.__navigateScoringGame(event):
+                    return
+            if self.clientDialog:
+                self.clientDialog.keyPressEvent(event)
         KXmlGuiWindow.keyPressEvent(self, event)
 
     def retranslateUi(self):
@@ -683,6 +648,7 @@ class PlayField(KXmlGuiWindow):
         self.actionScoring.setText(m18n("&Scoring"))
         self.actionScoreTable.setText(m18nc('kajongg', "&Score Table"))
         self.actionExplain.setText(m18n("&Explain Scores"))
+        self.actionAutoPlay.setText(m18n("&Demo Mode"))
 
     def changeEvent(self, event):
         """when the applicationwide language changes, recreate GUI"""
@@ -696,19 +662,16 @@ class PlayField(KXmlGuiWindow):
             self.playerWindow = PlayerList(self)
         self.playerWindow.show()
 
-    def selectGame(self):
+    def selectScoringGame(self):
         """show all games, select an existing game or create a new game"""
         gameSelector = Games(self)
         if gameSelector.exec_():
             selected = gameSelector.selectedGame
             if selected is not None:
-                Game.load(selected)
+                Game.load(selected, what=ScoringGame)
             else:
                 self.newGame()
             if self.game:
-                self.selectorBoard.fill(self.game)
-                if self.game.isScoringGame():
-                    self.centralView.scene().setFocusItem(self.selectorBoard.childItems()[0])
                 self.game.throwDices()
         gameSelector.close()
         self.refresh()
@@ -716,31 +679,28 @@ class PlayField(KXmlGuiWindow):
 
     def scoreGame(self):
         """score a local game"""
-        if self.selectGame():
+        if self.selectScoringGame():
             self.actionScoring.setChecked(True)
 
     def playGame(self):
         """play a remote game: log into a server and show its tables"""
         self.tableLists.append(TableList())
 
-    def abortGame(self, callback=None):
-        """aborts current game"""
-        msg = m18n("Do you really want to abort this game?")
-        if InternalParameters.autoPlay or self.game.finished() or \
-            KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes:
-            self.game.close(callback)
-            return True
-        else:
-            return False
-
     def adjustView(self):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
+        if not InternalParameters.scaleScene:
+            return
         if self.game:
-            if self.discardBoard:
-                self.discardBoard.scale()
-            if self.selectorBoard:
-                self.selectorBoard.scale()
+            with Animated(False):
+                self.game.wall.decorate()
+                if self.discardBoard:
+                    self.discardBoard.maximize()
+                if self.selectorBoard:
+                    self.selectorBoard.maximize()
+                for tile in self.game.wall.tiles:
+                    if tile.board:
+                        tile.board.placeTile(tile)
         view, scene = self.centralView, self.centralScene
         oldRect = view.sceneRect()
         view.setSceneRect(scene.itemsBoundingRect())
@@ -749,7 +709,7 @@ class PlayField(KXmlGuiWindow):
             view.fitInView(scene.itemsBoundingRect(), Qt.KeepAspectRatio)
 
     @apply
-    def tilesetName(): # pylint: disable-msg=E0202
+    def tilesetName(): # pylint: disable=E0202
         """the name of the current tileset"""
         def fget(self):
             return self.tileset.desktopFileName
@@ -758,7 +718,7 @@ class PlayField(KXmlGuiWindow):
         return property(**locals())
 
     @apply
-    def backgroundName(): # pylint: disable-msg=E0202
+    def backgroundName(): # pylint: disable=E0202
         """setting this also actually changes the background"""
         def fget(self):
             return self.background.desktopFileName if self.background else ''
@@ -771,31 +731,56 @@ class PlayField(KXmlGuiWindow):
 
     def applySettings(self):
         """apply preferences"""
-        self.settingsChanged = True
-        if self.tilesetName != common.PREF.tilesetName:
-            self.tilesetName = common.PREF.tilesetName
-            for item in self.centralScene.items():
-                if not isinstance(item, Tile): # shortcut
+        # pylint: disable=R0912
+        # too many branches
+        animate() # drain the queue
+        afterCurrentAnimationDo(self.__applySettings2)
+
+    def __applySettings2(self, dummyResults):
+        """now no animation is running"""
+        with Animated(False):
+            if self.tilesetName != common.PREF.tilesetName:
+                self.tilesetName = common.PREF.tilesetName
+                if self.game:
+                    self.game.wall.tileset = self.tileset
+                for item in self.centralScene.nonTiles():
                     try:
                         item.tileset = self.tileset
                     except AttributeError:
                         continue
-            # change players last because we need the wall already to be repositioned
+                # change players last because we need the wall already to be repositioned
+                self.adjustView() # the new tiles might be larger
             if self.game:
-                self.game.wall.decorate()
-            self.adjustView() # the new tiles might be larger
-        if self.game:
-            for player in self.game.players:
-                if player.handBoard:
-                    player.handBoard.rearrangeMelds = common.PREF.rearrangeMelds
-        if self.isVisible() and self.backgroundName != common.PREF.backgroundName:
-            self.backgroundName = common.PREF.backgroundName
+                for player in self.game.players:
+                    if player.handBoard:
+                        player.handBoard.rearrangeMelds = common.PREF.rearrangeMelds
+            if self.backgroundName != common.PREF.backgroundName:
+                self.backgroundName = common.PREF.backgroundName
+            if self.showShadows is None or self.showShadows != common.PREF.showShadows:
+                self.showShadows = common.PREF.showShadows
+                if self.game:
+                    wall = self.game.wall
+                    wall.showShadows = self.showShadows
+                self.selectorBoard.showShadows = self.showShadows
+                if self.discardBoard:
+                    self.discardBoard.showShadows = self.showShadows
+                for tile in self.centralScene.graphicsTileItems():
+                    tile.setClippingFlags()
+                self.adjustView()
         Sound.enabled = common.PREF.useSounds
+        self.centralScene.placeFocusRect()
 
     def showSettings(self):
         """show preferences dialog. If it already is visible, do nothing"""
         if KConfigDialog.showDialog("settings"):
             return
+        # if an animation is running, Qt segfaults somewhere deep
+        # in the SVG renderer rendering the wind tiles for the tile
+        # preview
+        afterCurrentAnimationDo(self.__showSettings2)
+
+    def __showSettings2(self, dummyResult):
+        """now that no animation is running, show settings dialog"""
         confDialog = ConfigDialog(self, "settings")
         self.connect(confDialog, SIGNAL('settingsChanged(QString)'),
            self.applySettings)
@@ -807,7 +792,7 @@ class PlayField(KXmlGuiWindow):
         selectDialog = SelectPlayers(self.game)
         if not selectDialog.exec_():
             return
-        return Game(selectDialog.names, selectDialog.cbRuleset.current)
+        return ScoringGame(selectDialog.names, selectDialog.cbRuleset.current)
 
     def __toggleWidget(self, checked):
         """user has toggled widget visibility with an action"""
@@ -831,12 +816,31 @@ class PlayField(KXmlGuiWindow):
             assert actionData
             actionData.hide()
 
+    def __toggleDemoMode(self, checked):
+        """switch on / off for autoPlay"""
+        if self.game:
+            self.centralScene.placeFocusRect() # show/hide it
+            if checked and self.clientDialog:
+                self.clientDialog.selectButton() # select default, abort timeout
+            self.game.autoPlay = checked
+        else:
+            InternalParameters.autoPlay = checked
+            if checked:
+                self.playGame()
+
     def scoringClosed(self):
         """the scoring window has been closed with ALT-F4 or similar"""
         self.actionScoring.setChecked(False)
 
     def nextHand(self):
         """save hand to database, update score table and balance in status line, prepare next hand"""
+        if self.game.winner:
+            for player in self.game.players:
+                player.usedDangerousFrom = None
+                for ruleBox in player.manualRuleBoxes:
+                    rule = ruleBox.rule
+                    if rule.name == 'Dangerous Game' and ruleBox.isChecked():
+                        self.game.winner.usedDangerousFrom = player
         self.game.saveHand()
         self.game.maybeRotateWinds()
         self.game.prepareHand()
@@ -846,8 +850,6 @@ class PlayField(KXmlGuiWindow):
         if not self.game:
             self.refresh()
             return
-        if not self.game.finished():
-            self.discardBoard.clear()
         self.refresh()
         self.game.wall.decorate()
 
@@ -861,10 +863,9 @@ class PlayField(KXmlGuiWindow):
         self.selectorBoard.setVisible(scoring)
         self.selectorBoard.setEnabled(scoring)
         self.discardBoard.setVisible(bool(game) and not scoring)
-        if game:
-            self.actionScoring.setEnabled(game is not None and game.roundsFinished < 4)
-        else:
-            self.actionScoring.setChecked(False)
+        self.actionScoring.setEnabled(scoring and not game.finished())
+        if self.actionScoring.isChecked():
+            self.actionScoring.setChecked(scoring and not game.finished())
         for view in [self.scoringDialog, self.explainView, self.scoreTable]:
             if view:
                 view.refresh(game)
@@ -873,16 +874,23 @@ class PlayField(KXmlGuiWindow):
     def changeAngle(self):
         """change the lightSource"""
         if self.game:
-            wall = self.game.wall
-            oldIdx = LIGHTSOURCES.index(wall.lightSource)
-            newLightSource = LIGHTSOURCES[(oldIdx + 1) % 4]
-            wall.lightSource = newLightSource
-            wall.decorate()
-        self.selectorBoard.lightSource = newLightSource
-        self.adjustView()
-        scoringDialog = self.actionScoring.data().toPyObject()
-        if isinstance(scoringDialog, ScoringDialog):
-            scoringDialog.computeScores()
+            afterCurrentAnimationDo(self.__changeAngle2)
+
+    def __changeAngle2(self, dummyResult):
+        """now that no animation is running, really change"""
+        if self.game: # might be finished meanwhile
+            with Animated(False):
+                wall = self.game.wall
+                oldIdx = LIGHTSOURCES.index(wall.lightSource)
+                newLightSource = LIGHTSOURCES[(oldIdx + 1) % 4]
+                wall.lightSource = newLightSource
+                self.selectorBoard.lightSource = newLightSource
+                self.discardBoard.lightSource = newLightSource
+                self.adjustView()
+                scoringDialog = self.actionScoring.data().toPyObject()
+                if isinstance(scoringDialog, ScoringDialog):
+                    scoringDialog.computeScores()
+                self.centralScene.placeFocusRect()
 
     def __showBalance(self):
         """show the player balances in the status bar"""
@@ -920,14 +928,3 @@ class PlayField(KXmlGuiWindow):
         """use this as a proxy such that module game does not have to import playfield.
         Game should also run on a server without KDE being installed"""
         return SwapDialog(swappers).exec_()
-
-    def hideAllFocusRect(self):
-        """hide all blue focus rects around tiles. There may be more than
-        one at the same time: The last discarded tile and a hidden tile of the player"""
-        if self.game:
-            boards = [x.handBoard for x in self.game.players]
-            boards.append(self.selectorBoard)
-            boards.append(self.discardBoard)
-            for board in boards:
-                if board:
-                    board.hideFocusRect()
