@@ -96,14 +96,14 @@ KBBMainWindow::KBBMainWindow()
 	// Menu "Game"
 	KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection());
 	m_pauseAction = KStandardGameAction::pause(this, SLOT(pause(bool)), actionCollection());
-	QAction* tutorial = actionCollection()->addAction("game_tutorial");
+	QAction* tutorial = actionCollection()->addAction( QLatin1String( "game_tutorial" ));
 	tutorial->setText(i18n("Start Tutorial"));
-	tutorial->setIcon(KIcon("footprint"));
+	tutorial->setIcon(KIcon( QLatin1String( "footprint" )));
 	tutorial->setToolTip(i18n("Start tutorial"));
 	tutorial->setWhatsThis(i18n("<qt>The <b>tutorial</b> is a fast, user friendly and interactive way to learn the rules of the game. Start it if you do not know them!</qt>"));
 	connect(tutorial, SIGNAL(triggered(bool)), SLOT(startTutorial()));
 	KStandardGameAction::quit(this, SLOT(close()), actionCollection());
-	QAction* sandbox = actionCollection()->addAction("game_sandbox");
+	QAction* sandbox = actionCollection()->addAction( QLatin1String( "game_sandbox" ));
 	sandbox->setText(i18n("New Sandbox Game"));
 	sandbox->setToolTip(i18n("Start a new sandbox game"));
 	sandbox->setWhatsThis(i18n("<qt><p>In a <b>sandbox game</b>, the solution is displayed at the beginning of the game. This is useful to understand the game principles.</p><p>However: after a while, it is not really fun and you should try to start a real game!</p></qt>"));
@@ -111,10 +111,10 @@ KBBMainWindow::KBBMainWindow()
 	KStandardGameAction::highscores(this, SLOT(showHighscores()), actionCollection());
 
 	// Menu "Move"
-	m_doneAction = actionCollection()->addAction("move_done");
+	m_doneAction = actionCollection()->addAction( QLatin1String( "move_done" ));
 	m_doneAction->setText(i18nc("This is the last action of a game to check the result, when the user is done.", "Done!"));
-	m_doneAction->setWhatsThis(i18n("<qt><ul><li>First, you have to place all the balls on the black box. To guess the right possitions of the balls and see how they interact with laser beams, you should use the lasers that are disposed around the black box.</li><li><b>When you think you are done</b>, you should click here.</li></ul><p>Note that it is possible to click here only if you placed the right number of balls.</p></qt>"));
-	m_doneAction->setIcon(KIcon("dialog-ok"));
+	m_doneAction->setWhatsThis(i18n("<qt><ul><li>First, you have to place all the balls on the black box. To guess the correct positions of the balls and see how they interact with laser beams, you should use the lasers that are positioned around the black box.</li><li><b>When you think you are done</b>, you should click here.</li></ul><p>Note that it is only possible to click here if you have placed the correct number of balls.</p></qt>"));
+	m_doneAction->setIcon(KIcon( QLatin1String( "dialog-ok" )));
 	connect(m_doneAction, SIGNAL(triggered(bool)), SLOT(done()));
 	m_solveAction = KStandardGameAction::solve(this, SLOT(solve()), actionCollection());
 	m_solveAction->setToolTip(i18n("Give up the game"));
@@ -157,37 +157,37 @@ KBBMainWindow::KBBMainWindow()
 
 
 	// Keyboard only
-	QAction* action = actionCollection()->addAction( "move_down" );
+	KAction* action = actionCollection()->addAction( QLatin1String(  "move_down" ) );
 	action->setText( i18n("Move Down") );
 	connect(action, SIGNAL(triggered(bool) ), m_gameWidget, SLOT(keyboardMoveDown()));
 	action->setShortcut(Qt::Key_Down);
 	addAction(action);
 	
-	action = actionCollection()->addAction( "move_up" );
+	action = actionCollection()->addAction( QLatin1String(  "move_up" ) );
 	action->setText( i18n("Move Up") );
 	connect(action, SIGNAL(triggered(bool) ), m_gameWidget, SLOT(keyboardMoveUp()));
 	action->setShortcut(Qt::Key_Up);
 	addAction(action);
 	
-	action = actionCollection()->addAction( "move_left" );
+	action = actionCollection()->addAction( QLatin1String(  "move_left" ) );
 	action->setText( i18n("Move Left") );
 	connect(action, SIGNAL(triggered(bool) ), m_gameWidget, SLOT(keyboardMoveLeft()));
 	action->setShortcut(Qt::Key_Left);
 	addAction(action);
 	
-	action = actionCollection()->addAction( "move_right" );
+	action = actionCollection()->addAction( QLatin1String(  "move_right" ) );
 	action->setText( i18n("Move Right") );
 	connect(action, SIGNAL(triggered(bool) ), m_gameWidget, SLOT(keyboardMoveRight()));
 	action->setShortcut(Qt::Key_Right);
 	addAction(action);
 	
-	action = actionCollection()->addAction("switch_ball");
+	action = actionCollection()->addAction( QLatin1String( "switch_ball" ));
 	action->setText(i18n("Switch Ball or Shoot Laser"));
 	connect(action, SIGNAL(triggered(bool)), m_gameWidget, SLOT(keyboardEnter()));
 	action->setShortcut(Qt::Key_Return);
 	addAction(action);
 	
-	action = actionCollection()->addAction("switch_marker");
+	action = actionCollection()->addAction( QLatin1String( "switch_marker" ));
 	action->setText(i18n("Switch Marker"));
 	connect(action, SIGNAL(triggered(bool)), m_gameWidget, SLOT(keyboardSpace()));
 	action->setShortcut(Qt::Key_Space);
@@ -260,7 +260,7 @@ void KBBMainWindow::levelChanged(KGameDifficulty::standardLevel level)
 			m_rows = 12;
 			break;
 		case KGameDifficulty::Configurable:
-			m_gameWidget->popupText(i18nc("The text may not be too wide. So please use some HTML-BR-tags to have something more or less as wide as in english. Thanks!", "Note: You can change<br />the parameters of<br />custom games in the<br />setting dialog."));
+			m_gameWidget->popupText(i18nc("The text may not be too wide. So please use some HTML-BR-tags to have something more or less as wide as in english. Thanks!", "Note: You can change<br />the parameters of<br />custom games in the<br />Settings dialog."));
 			break;
 	}
 
@@ -296,7 +296,7 @@ void KBBMainWindow::updateStats()
 	if (ballsLeftToPlace<0)
 		m_doneAction->setToolTip(i18np("First, you need to remove 1 ball from the black box.", "First, you need to remove %1 balls from the black box.", -ballsLeftToPlace));
 	else if (ballsLeftToPlace==0) {
-		m_doneAction->setToolTip(i18n("To check if you successful guessed the ball positions, click here!"));
+		m_doneAction->setToolTip(i18n("To check if you successfully guessed the ball positions, click here!"));
 	} else if (ballsLeftToPlace>0) {
 		m_doneAction->setToolTip(i18np("You need to place 1 more ball on the black box.", "You need to place %1 more balls on the black box.", ballsLeftToPlace));
 	}
@@ -313,9 +313,9 @@ void KBBMainWindow::updateStats()
 	if (!m_tutorial->isVisible()) {
 		if (m_boardEnabled) {
 			if (ballsLeftToPlace<0) {
-				statusBar()->changeItem((i18np("1 ball too much!", "%1 balls too much!", -ballsLeftToPlace)), SRUN);
+				statusBar()->changeItem((i18np("1 ball too many!", "%1 balls too many!", -ballsLeftToPlace)), SRUN);
 			} else if (ballsLeftToPlace==0) {
-				statusBar()->changeItem(i18n("No more ball to place"), SRUN);
+				statusBar()->changeItem(i18n("No more balls to place"), SRUN);
 			} else if (ballsLeftToPlace>0) {
 				statusBar()->changeItem(i18np("1 ball to place", "%1 balls to place", ballsLeftToPlace), SRUN);
 			}	
@@ -357,6 +357,7 @@ void KBBMainWindow::done()
 		if ((!m_tutorial->isVisible()) && (!m_sandboxMode) && (KGameDifficulty::level() != KGameDifficulty::Configurable) && (score<KBBGameDoc::SCORE_LOST)) {
 			KScoreDialog scoreDialog(KScoreDialog::Score | KScoreDialog::Name, this);
 			scoreDialog.addLocalizedConfigGroupNames(KGameDifficulty::localizedLevelStrings());
+			scoreDialog.setConfigGroupWeights(KGameDifficulty::levelWeights());
 			scoreDialog.setConfigGroup(KGameDifficulty::localizedLevelString());
 	
 			KScoreDialog::FieldInfo scoreInfo;
@@ -428,6 +429,7 @@ void KBBMainWindow::showHighscores()
 {
 	KScoreDialog scoreDialog(KScoreDialog::Score | KScoreDialog::Name, this);
 	scoreDialog.addLocalizedConfigGroupNames(KGameDifficulty::localizedLevelStrings());
+	scoreDialog.setConfigGroupWeights(KGameDifficulty::levelWeights());
 	scoreDialog.setConfigGroup( KGameDifficulty::localizedLevelString() );
 	scoreDialog.exec();
 }
@@ -436,10 +438,10 @@ void KBBMainWindow::showHighscores()
 void KBBMainWindow::solve()
 {
 	if (m_tutorial->isVisible() && !m_tutorial->maySolve()) {
-		KMessageBox::sorry(this, i18n("Sorry, you may not give up the tutorial."), i18n("Solve"));
+		KMessageBox::sorry(this, i18n("Sorry, you may not give up during the tutorial."), i18n("Solve"));
 	} else {
 		if (m_gameDoc->numberOfBallsToPlace()==m_gameDoc->numberOfBallsPlaced()) {
-			if (KMessageBox::warningContinueCancel(this, i18n("You placed all the balls. Great!\nYou should now click on \"Done!\" to end the game and check if you guessed right.\nSo, do you really want to give up this game?"), QString(), KGuiItem(i18n("Give up"))) == KMessageBox::Continue)
+			if (KMessageBox::warningContinueCancel(this, i18n("You placed all the balls. Great!\nYou should now click on \"Done!\" to end the game and check if you guessed correctly.\nSo, do you really want to give up this game?"), QString(), KGuiItem(i18n("Give up"))) == KMessageBox::Continue)
 				solving();
 		} else if (KMessageBox::warningContinueCancel(this, i18np("You should place %1 ball!\n", "You should place %1 balls!\n", m_gameDoc->numberOfBallsToPlace()) + i18np("You have placed %1.\n", "You have placed %1.\n", m_gameDoc->numberOfBallsPlaced()) + i18n("Do you really want to give up this game?"), QString(), KGuiItem(i18n("Give up"))) == KMessageBox::Continue)
 			solving();
