@@ -1,5 +1,6 @@
 /*
     Copyright 2007-2009 Fela Winkelmolen <fela.kde@gmail.com> 
+    Copyright 2010 Brian Croom <brian.s.croom@gmail.com>
   
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +20,6 @@
 #include "gameengine.h"
 #include "canvasitems.h"
 #include "canvaswidget.h"
-#include "renderer.h"
 #include "ui_generalsettings.h"
 #include "settings.h"
 
@@ -59,10 +59,13 @@ private:
 
 MainWindow::MainWindow(QWidget *parent) 
     : KXmlGuiWindow(parent),
-      canvasWidget(new CanvasWidget(this))
+      renderer(Settings::defaultThemeValue()),
+      canvasWidget(new CanvasWidget(&renderer, this))
 {
+    renderer.setTheme(Settings::theme());
     // TODO: find a better way..
     Item::setCanvas(canvasWidget);
+    Item::setRenderer(&renderer);
     new Background; // the background put's itself into the canvasWidget
     gameEngine = new GameEngine(this); // must be called after Item::setCanvas()
     
@@ -137,9 +140,9 @@ void MainWindow::setupActions()
     KAction *fireAction = new KAction(this);
     fireAction->setText(i18n("Fire the ball"));
     fireAction->setShortcut(Qt::Key_Space);
-    fireAction->setIcon(KIcon("kbreakout"));
+    fireAction->setIcon(KIcon( QLatin1String( "kbreakout" )));
     connect(fireAction, SIGNAL(triggered()), this, SLOT(fire()));
-    actionCollection()->addAction("fire", fireAction);
+    actionCollection()->addAction( QLatin1String( "fire" ), fireAction);
 
     pauseAction = KStandardGameAction::pause(this,
               SLOT(setGamePaused(bool)), actionCollection());
@@ -181,13 +184,7 @@ void MainWindow::configureSettings()
 
 void MainWindow::loadSettings() 
 {
-    if (!Renderer::self()->loadTheme(Settings::theme())) {
-        KMessageBox::error(this,  
-           i18n("Failed to load \"%1\" theme. Please check your installation.",
-           Settings::theme()));
-        return;
-    }
-    
+    renderer.setTheme(Settings::theme());
     canvasWidget->reloadSprites();
 }
 
