@@ -39,8 +39,6 @@
 #include "dealerinfo.h"
 #include "patsolve/clocksolver.h"
 
-#include "Shuffle"
-
 #include <KLocale>
 
 
@@ -48,7 +46,7 @@ void Clock::initialize()
 {
     setSceneAlignment( AlignHCenter | AlignVCenter );
 
-    static_cast<KStandardCardDeck*>( deck() )->setDeckContents();
+    setDeckContents();
 
     const qreal dist_x = 1.11;
     const qreal ys[12] = {   0./96,  15./96,  52./96, 158./96, 264./96, 301./96, 316./96, 301./96, 264./96, 158./96,  52./96,  15./96};
@@ -69,7 +67,7 @@ void Clock::initialize()
         store[i] = new PatPile( this, 14 + i, QString("store%1").arg(i) );
         store[i]->setPileRole(PatPile::Tableau);
         store[i]->setLayoutPos(dist_x*(i%4), 2.5 * (i/4));
-        store[i]->setReservedSpace( 0, 0, 1.0, 2.3 );
+        store[i]->setBottomPadding( 1.3 );
         store[i]->setKeyboardSelectHint( KCardPile::AutoFocusTop );
         store[i]->setKeyboardDropHint( KCardPile::AutoFocusTop );
     }
@@ -78,26 +76,25 @@ void Clock::initialize()
     setSolver( new ClockSolver( this ) );
 }
 
-void Clock::restart()
+void Clock::restart( const QList<KCard*> & cards )
 {
-    static const KStandardCardDeck::Suit suits[12] = { KStandardCardDeck::Diamonds, KStandardCardDeck::Spades, KStandardCardDeck::Hearts, KStandardCardDeck::Clubs,
-                                                       KStandardCardDeck::Diamonds, KStandardCardDeck::Spades, KStandardCardDeck::Hearts, KStandardCardDeck::Clubs,
-                                                       KStandardCardDeck::Diamonds, KStandardCardDeck::Spades, KStandardCardDeck::Hearts, KStandardCardDeck::Clubs };
-    static const KStandardCardDeck::Rank ranks[12] = { KStandardCardDeck::Nine, KStandardCardDeck::Ten, KStandardCardDeck::Jack, KStandardCardDeck::Queen,
-                                                       KStandardCardDeck::King, KStandardCardDeck::Two, KStandardCardDeck::Three, KStandardCardDeck::Four,
-                                                       KStandardCardDeck::Five, KStandardCardDeck::Six, KStandardCardDeck::Seven, KStandardCardDeck::Eight };
-
-    QList<KCard*> cards = shuffled( deck()->cards(), gameNumber() );
+    static const KCardDeck::Suit suits[12] = { KCardDeck::Diamonds, KCardDeck::Spades, KCardDeck::Hearts, KCardDeck::Clubs,
+                                               KCardDeck::Diamonds, KCardDeck::Spades, KCardDeck::Hearts, KCardDeck::Clubs,
+                                               KCardDeck::Diamonds, KCardDeck::Spades, KCardDeck::Hearts, KCardDeck::Clubs };
+    static const KCardDeck::Rank ranks[12] = { KCardDeck::Nine, KCardDeck::Ten, KCardDeck::Jack, KCardDeck::Queen,
+                                               KCardDeck::King, KCardDeck::Two, KCardDeck::Three, KCardDeck::Four,
+                                               KCardDeck::Five, KCardDeck::Six, KCardDeck::Seven, KCardDeck::Eight };
 
     const QPointF center = ( target[0]->pos() + target[6]->pos() ) / 2;
 
     int j = 0;
-    while ( !cards.isEmpty() )
+    QList<KCard*> cardList = cards;
+    while ( !cardList.isEmpty() )
     {
-        KCard * c = cards.takeLast();
+        KCard * c = cardList.takeLast();
         for ( int i = 0; i < 12; ++i )
         {
-            if ( getRank( c ) == ranks[i] && getSuit( c ) == suits[i] )
+            if ( c->rank() == ranks[i] && c->suit() == suits[i] )
             {
                 QPointF initPos = (2 * center + target[(i + 2) % 12]->pos()) / 3;
                 addCardForDeal( target[i], c, true, initPos );
@@ -127,14 +124,14 @@ bool Clock::checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const
     if ( pile->pileRole() == PatPile::Tableau )
     {
         return oldCards.isEmpty()
-               || getRank( newCards.first() ) == getRank( oldCards.last() ) - 1;
+               || newCards.first()->rank() == oldCards.last()->rank() - 1;
     }
     else
     {
-        return getSuit( oldCards.last() ) == getSuit( newCards.first() )
-               && ( getRank( newCards.first() ) == getRank( oldCards.last() ) + 1
-                    || ( getRank( oldCards.last() ) == KStandardCardDeck::King
-                         && getRank( newCards.first() ) == KStandardCardDeck::Ace ) );
+        return oldCards.last()->suit() == newCards.first()->suit()
+               && ( newCards.first()->rank() == oldCards.last()->rank() + 1
+                    || ( oldCards.last()->rank() == KCardDeck::King
+                         && newCards.first()->rank() == KCardDeck::Ace ) );
     }
 }
 
