@@ -42,8 +42,6 @@
 #include "speeds.h"
 #include "patsolve/freecellsolver.h"
 
-#include "Shuffle"
-
 #include <KDebug>
 #include <KLocale>
 
@@ -55,7 +53,7 @@ void Freecell::initialize()
 {
     startMovingQueued = false;
 
-    static_cast<KStandardCardDeck*>( deck() )->setDeckContents();
+    setDeckContents();
 
     const qreal topRowDist = 1.08;
     const qreal bottomRowDist = 1.13;
@@ -75,7 +73,8 @@ void Freecell::initialize()
         store[i] = new PatPile( this, 1 + i, QString( "store%1" ).arg( i ) );
         store[i]->setPileRole(PatPile::Tableau);
         store[i]->setLayoutPos( bottomRowDist * i, 1.3 );
-        store[i]->setReservedSpace( 0, 0, 1, 3.5 );
+        store[i]->setBottomPadding( 2.5 );
+        store[i]->setHeightPolicy( KCardPile::GrowDown );
         store[i]->setKeyboardSelectHint( KCardPile::AutoFocusDeepestRemovable );
         store[i]->setKeyboardDropHint( KCardPile::AutoFocusTop );
     }
@@ -96,14 +95,14 @@ void Freecell::initialize()
 }
 
 
-void Freecell::restart()
+void Freecell::restart( const QList<KCard*> & cards )
 {
-    QList<KCard*> cards = shuffled(deck()->cards(), gameNumber() );
+    QList<KCard*> cardList = cards;
 
     int column = 0;
-    while ( !cards.isEmpty() )
+    while ( !cardList.isEmpty() )
     {
-        addCardForDeal( store[column], cards.takeLast(), true, store[0]->pos() );
+        addCardForDeal( store[column], cardList.takeLast(), true, store[0]->pos() );
         column = (column + 1) % 8;
     }
 
@@ -312,8 +311,8 @@ bool Freecell::canPutStore(const PatPile *c1, const QList<KCard*> &c2) const
     KCard *c = c2.first(); // we assume there are only valid sequences
 
     // ok if in sequence, alternate colors
-    return getRank( c1->top() ) == getRank( c ) + 1
-           && getIsRed( c1->top() ) != getIsRed( c );
+    return c1->top()->rank() == c->rank() + 1
+           && c1->top()->color() != c->color();
 }
 
 bool Freecell::checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards) const
