@@ -43,8 +43,6 @@
 #include "speeds.h"
 #include "patsolve/spidersolver.h"
 
-#include "Shuffle"
-
 #include <KLocale>
 #include <KRandom>
 #include <KSelectAction>
@@ -85,7 +83,8 @@ void Spider::initialize()
         stack[column]->setLayoutPos(dist_x * column, 0);
         stack[column]->setZValue(20);
         stack[column]->setAutoTurnTop(true);
-        stack[column]->setReservedSpace( 0, 0, 1, 2.5 );
+        stack[column]->setBottomPadding( 1.5 );
+        stack[column]->setHeightPolicy( KCardPile::GrowDown );
         stack[column]->setKeyboardSelectHint( KCardPile::AutoFocusDeepestRemovable );
         stack[column]->setKeyboardDropHint( KCardPile::AutoFocusTop );
     }
@@ -193,15 +192,15 @@ void Spider::createDeck()
     // suits, while the new logic is more flexible. We maintain the card
     // ordering by always passing a list of 4 suits even if we really only
     // have one or two.
-    QList<KStandardCardDeck::Suit> suits;
+    QList<KCardDeck::Suit> suits;
     if ( m_suits == 1 )
-        suits << KStandardCardDeck::Spades << KStandardCardDeck::Spades << KStandardCardDeck::Spades << KStandardCardDeck::Spades;
+        suits << KCardDeck::Spades << KCardDeck::Spades << KCardDeck::Spades << KCardDeck::Spades;
     else if ( m_suits == 2 )
-        suits << KStandardCardDeck::Hearts << KStandardCardDeck::Spades << KStandardCardDeck::Hearts << KStandardCardDeck::Spades;
+        suits << KCardDeck::Hearts << KCardDeck::Spades << KCardDeck::Hearts << KCardDeck::Spades;
     else
-        suits << KStandardCardDeck::Clubs << KStandardCardDeck::Diamonds << KStandardCardDeck::Hearts << KStandardCardDeck::Spades;
+        suits << KCardDeck::Clubs << KCardDeck::Diamonds << KCardDeck::Hearts << KCardDeck::Spades;
 
-    static_cast<KStandardCardDeck*>( deck() )->setDeckContents( 2, suits );
+    setDeckContents( 2, suits );
 }
 
 
@@ -212,7 +211,7 @@ bool Spider::checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, cons
     // on any suit card of one higher rank
     return pile->pileRole() == PatPile::Tableau
            && ( oldCards.isEmpty()
-                || getRank( oldCards.last() ) == getRank( newCards.first() ) + 1 );
+                || oldCards.last()->rank() == newCards.first()->rank() + 1 );
 }
 
 
@@ -264,10 +263,8 @@ void Spider::setGameOptions(const QString& options)
 }
 
 
-void Spider::restart()
+void Spider::restart( const QList<KCard*> & cards )
 {
-    clearHighlightedItems();
-
     // make the redeal piles visible
     for (int i = 0; i < 5; ++i )
         redeals[i]->setVisible( true );
@@ -281,25 +278,25 @@ void Spider::restart()
     m_leg = 0;
     m_redeal = 0;
 
-    QList<KCard*> cards = shuffled( deck()->cards(), gameNumber() );
+    QList<KCard*> cardList = cards;
 
     int column = 0;
     // deal face down cards (5 to first 4 piles, 4 to last 6)
     for ( int i = 0; i < 44; ++i )
     {
-        addCardForDeal( stack[column], cards.takeLast(), false, randomPos() );
+        addCardForDeal( stack[column], cardList.takeLast(), false, randomPos() );
         column = (column + 1) % 10;
     }
     // deal face up cards, one to each pile
     for ( int i = 0; i < 10; ++i )
     {
-        addCardForDeal( stack[column], cards.takeLast(), true, randomPos() );
+        addCardForDeal( stack[column], cardList.takeLast(), true, randomPos() );
         column = (column + 1) % 10;
     }
     // deal the remaining cards into 5 'redeal' piles
     for ( int column = 0; column < 5; ++column )
         for ( int i = 0; i < 10; ++i )
-            addCardForDeal( redeals[column], cards.takeLast(), false, randomPos() );
+            addCardForDeal( redeals[column], cardList.takeLast(), false, randomPos() );
 
     startDealAnimation();
 
@@ -311,19 +308,21 @@ QList<KCard*> Spider::getRun(KCard *c) const
 {
     QList<KCard*> result;
 
-    PatPile *p = dynamic_cast<PatPile*>( c->pile() );
+    KCardPile * p = c->pile();
     if (!p || p->isEmpty())
         return result;
 
     result.append(c);
 
-    KStandardCardDeck::Suit s = getSuit( c );
-    int v = getRank( c );
+    int suit = c->suit();
+    int rank = c->rank();
 
     int index = p->indexOf(c);
     c = p->at(--index);
-    while (index >= 0 && c->isFaceUp()
-           && getSuit( c ) == s && getRank( c ) == ++v)
+    while ( index >= 0
+            && c->isFaceUp()
+            && c->suit() == suit
+            && c->rank() == ++rank )
     {
         result.prepend(c);
         c = p->at(--index);
@@ -345,8 +344,8 @@ void Spider::moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duratio
     // then it could have a full deck that needs removed.
     if ( p
          && p->pileRole() != PatPile::Foundation
-         && getRank( cards.last() ) == KStandardCardDeck::Ace
-         && getSuit( cards.first() ) == getSuit( p->top() )
+         && cards.last()->rank() == KCardDeck::Ace
+         && cards.first()->suit() == p->top()->suit()
          && p->count() > 12 )
     {
         checkPileDeck( p );
@@ -362,10 +361,12 @@ bool Spider::checkPileDeck( KCardPile * pile, bool checkForDemo )
     if (pile->isEmpty())
         return false;
 
-    if ( getRank( pile->top() ) == KStandardCardDeck::Ace) {
+    if ( pile->top()->rank() == KCardDeck::Ace )
+    {
         // just using the CardList to see if this goes to King
         QList<KCard*> run = getRun(pile->top());
-        if ( getRank( run.first() ) == KStandardCardDeck::King) {
+        if ( run.first()->rank() == KCardDeck::King )
+        {
             PatPile *leg = legs[m_leg];
             leg->setVisible(true);
             relayoutPiles( DURATION_RELAYOUT );
@@ -479,30 +480,15 @@ public:
     SpideDealerInfo()
       : DealerInfo(I18N_NOOP("Spider"), SpiderGeneralId)
     {
-        addId(SpiderOneSuitId);
-        addId(SpiderTwoSuitId);
-        addId(SpiderFourSuitId);
+        addSubtype( SpiderOneSuitId, I18N_NOOP( "Spider (1 Suit)" ) );
+        addSubtype( SpiderTwoSuitId, I18N_NOOP( "Spider (2 Suit)" ) );
+        addSubtype( SpiderFourSuitId, I18N_NOOP( "Spider (4 Suit)" ) );
     }
 
     virtual DealerScene *createGame() const
     {
         return new Spider();
     }
-
-    virtual const char * name( int id ) const
-    {
-        switch ( id )
-        {
-        case DealerInfo::SpiderOneSuitId :
-            return I18N_NOOP("Spider (1 Suit)");
-        case DealerInfo::SpiderTwoSuitId :
-            return I18N_NOOP("Spider (2 Suit)");
-        case DealerInfo::SpiderFourSuitId :
-            return I18N_NOOP("Spider (4 Suit)");
-        default :
-            return m_name;
-        }
-    }
 } spideDealerInfo;
 
 
