@@ -55,6 +55,7 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <signal.h>
+#include <qcache.h>
 
 namespace Ksirk{
 namespace GameLogic {
@@ -199,10 +200,7 @@ GameAutomaton::GameAutomaton() :
 GameAutomaton::~GameAutomaton()
 {
   kDebug();
-  foreach (Goal* goal, m_goals)
-  {
-    delete goal;
-  }
+  qDeleteAll(m_goals);
 }
 
 void GameAutomaton::init(KGameWindow* gw)
@@ -1188,7 +1186,7 @@ bool GameAutomaton::joinNetworkGame()
 {
   kDebug();
    if (stateName() == "INIT"
-     || (KMessageBox::warningContinueCancel(m_game,i18n("Do you really want to end your current game and join another?"),i18n("New game confirmation"),KStandardGuiItem::yes()) == KMessageBox::Continue))
+     || (KMessageBox::warningContinueCancel(m_game,i18n("Do you really want to end your current game and join another?"),i18n( "New game confirmation" ),KStandardGuiItem::yes()) == KMessageBox::Continue))
    {
       m_game->joinNetworkGame();
    }
@@ -1216,7 +1214,7 @@ bool GameAutomaton::connectToServ()
 
 bool GameAutomaton::joinJabberGame(const QString& nick)
 {
-  if (stateName() == "INIT" || (KMessageBox::warningContinueCancel(m_game,i18n("Do you really want to end your current game and join another?"),i18n("New game confirmation"),KStandardGuiItem::yes()) == KMessageBox::Continue))
+  if (stateName() == "INIT" || (KMessageBox::warningContinueCancel(m_game,i18n("Do you really want to end your current game and join another?"),i18n( "New game confirmation" ),KStandardGuiItem::yes()) == KMessageBox::Continue))
   {
     // stop game
     setGameStatus(KGame::End);
@@ -1887,6 +1885,7 @@ void GameAutomaton::movingArmyArrived(Country* country, unsigned int number)
   kDebug() << number << endl;
   country->incrNbArmies(number);
   country->createArmiesSprites();
+  checkGoal(country->owner());
 }
 
 void GameAutomaton::firingFinished()
@@ -2627,7 +2626,7 @@ void GameAutomaton::actionNextPlayer()
     QDataStream stream(&buffer, QIODevice::WriteOnly);
     stream << currentPlayer()->name();
     stream << (quint32)NEWARMIES;
-    kDebug() << "sending NextPlayerNormal" << currentPlayer()->name() << NEWARMIES;
+    kDebug() << "(state " << stateName() << ") sending NextPlayerNormal" << currentPlayer()->name() << NEWARMIES;
     sendMessage(buffer,NextPlayerNormal);
     m_game-> cancelAction();
   }
@@ -2637,10 +2636,7 @@ void GameAutomaton::removeAllPlayers()
 {
   kDebug();
   m_currentPlayer = "";
-  foreach (KPlayer*p, *playerList())
-  {
-    delete p;
-  }
+  qDeleteAll(*playerList());
   playerList()->clear();
 }
 
@@ -2650,7 +2646,7 @@ void GameAutomaton::newGameNext()
   m_startingGame = true;
   state(INIT);
   
-  kDebug() << "Changing skin" << endl;
+  kDebug() << "Changing skin";
   m_skin = m_game->newGameSetup()->skin();
   if (m_game->newGameSetup()->networkGameType() == Socket)
   {
@@ -2659,6 +2655,19 @@ void GameAutomaton::newGameNext()
   m_game->finishSetupPlayers();
 }
 
+void GameAutomaton::checkGoal(Player* player) 
+{
+  QByteArray buffer;
+  QDataStream stream(&buffer, QIODevice::WriteOnly);
+  if (player == 0)
+    stream << currentPlayer()->id();
+  else
+    stream << player->id();
+  kDebug() << "sending CheckGoal";
+  sendMessage(buffer,CheckGoal);
+}
+
+
 } // closing namespace GameLogic
 } // closing namespace Ksirk
 
