@@ -32,7 +32,7 @@
 #include <QFileInfo>
 #include <QDir>
 
-#include <ksvgrenderer.h>
+#include <qsvgrenderer.h>
 #include <kpixmapcache.h>
 #include <kconfig.h>
 #include <kstandarddirs.h>
@@ -131,124 +131,138 @@ bool KCardInfo::operator==( const KCardInfo& c ) const
 
 QString KCardInfo::pngName() const
 {
-    if( suit() == Club    && card() == Ace ) return "1.png";
-    if( suit() == Spade   && card() == Ace ) return "2.png";
-    if( suit() == Heart   && card() == Ace ) return "3.png";
-    if( suit() == Diamond && card() == Ace ) return "4.png";
-    if( suit() == Club    && card() == King ) return "5.png";
-    if( suit() == Spade   && card() == King ) return "6.png";
-    if( suit() == Heart   && card() == King ) return "7.png";
-    if( suit() == Diamond && card() == King ) return "8.png";
-    if( suit() == Club    && card() == Queen ) return "9.png";
-    if( suit() == Spade   && card() == Queen ) return "10.png";
-    if( suit() == Heart   && card() == Queen ) return "11.png";
-    if( suit() == Diamond && card() == Queen ) return "12.png";
-    if( suit() == Club    && card() == Jack ) return "13.png";
-    if( suit() == Spade   && card() == Jack ) return "14.png";
-    if( suit() == Heart   && card() == Jack ) return "15.png";
-    if( suit() == Diamond && card() == Jack ) return "16.png";
-    if( suit() == Club    && card() == Ten ) return "17.png";
-    if( suit() == Spade   && card() == Ten ) return "18.png";
-    if( suit() == Heart   && card() == Ten ) return "19.png";
-    if( suit() == Diamond && card() == Ten ) return "20.png";
-    if( suit() == Club    && card() == Nine ) return "21.png";
-    if( suit() == Spade   && card() == Nine ) return "22.png";
-    if( suit() == Heart   && card() == Nine ) return "23.png";
-    if( suit() == Diamond && card() == Nine ) return "24.png";
-    if( suit() == Club    && card() == Eight ) return "25.png";
-    if( suit() == Spade   && card() == Eight ) return "26.png";
-    if( suit() == Heart   && card() == Eight ) return "27.png";
-    if( suit() == Diamond && card() == Eight ) return "28.png";
-    if( suit() == Club    && card() == Seven ) return "29.png";
-    if( suit() == Spade   && card() == Seven ) return "30.png";
-    if( suit() == Heart   && card() == Seven ) return "31.png";
-    if( suit() == Diamond && card() == Seven ) return "32.png";
-    if( suit() == Club    && card() == Six ) return "33.png";
-    if( suit() == Spade   && card() == Six ) return "34.png";
-    if( suit() == Heart   && card() == Six ) return "35.png";
-    if( suit() == Diamond && card() == Six ) return "36.png";
-    if( suit() == Club    && card() == Five ) return "37.png";
-    if( suit() == Spade   && card() == Five ) return "38.png";
-    if( suit() == Heart   && card() == Five ) return "39.png";
-    if( suit() == Diamond && card() == Five ) return "40.png";
-    if( suit() == Club    && card() == Four ) return "41.png";
-    if( suit() == Spade   && card() == Four ) return "42.png";
-    if( suit() == Heart   && card() == Four ) return "43.png";
-    if( suit() == Diamond && card() == Four ) return "44.png";
-    if( suit() == Club    && card() == Three ) return "45.png";
-    if( suit() == Spade   && card() == Three ) return "46.png";
-    if( suit() == Heart   && card() == Three ) return "47.png";
-    if( suit() == Diamond && card() == Three ) return "48.png";
-    if( suit() == Club    && card() == Two ) return "49.png";
-    if( suit() == Spade   && card() == Two ) return "50.png";
-    if( suit() == Heart   && card() == Two ) return "51.png";
-    if( suit() == Diamond && card() == Two ) return "52.png";
-    return "";
+    if( suit() == Club    && card() == Ace ) return QLatin1String( "1.png" );
+    if( suit() == Spade   && card() == Ace ) return QLatin1String( "2.png" );
+    if( suit() == Heart   && card() == Ace ) return QLatin1String( "3.png" );
+    if( suit() == Diamond && card() == Ace ) return QLatin1String( "4.png" );
+    if( suit() == Club    && card() == King ) return QLatin1String( "5.png" );
+    if( suit() == Spade   && card() == King ) return QLatin1String( "6.png" );
+    if( suit() == Heart   && card() == King ) return QLatin1String( "7.png" );
+    if( suit() == Diamond && card() == King ) return QLatin1String( "8.png" );
+    if( suit() == Club    && card() == Queen ) return QLatin1String( "9.png" );
+    if( suit() == Spade   && card() == Queen ) return QLatin1String( "10.png" );
+    if( suit() == Heart   && card() == Queen ) return QLatin1String( "11.png" );
+    if( suit() == Diamond && card() == Queen ) return QLatin1String( "12.png" );
+    if( suit() == Club    && card() == Jack ) return QLatin1String( "13.png" );
+    if( suit() == Spade   && card() == Jack ) return QLatin1String( "14.png" );
+    if( suit() == Heart   && card() == Jack ) return QLatin1String( "15.png" );
+    if( suit() == Diamond && card() == Jack ) return QLatin1String( "16.png" );
+    if( suit() == Club    && card() == Ten ) return QLatin1String( "17.png" );
+    if( suit() == Spade   && card() == Ten ) return QLatin1String( "18.png" );
+    if( suit() == Heart   && card() == Ten ) return QLatin1String( "19.png" );
+    if( suit() == Diamond && card() == Ten ) return QLatin1String( "20.png" );
+    if( suit() == Club    && card() == Nine ) return QLatin1String( "21.png" );
+    if( suit() == Spade   && card() == Nine ) return QLatin1String( "22.png" );
+    if( suit() == Heart   && card() == Nine ) return QLatin1String( "23.png" );
+    if( suit() == Diamond && card() == Nine ) return QLatin1String( "24.png" );
+    if( suit() == Club    && card() == Eight ) return QLatin1String( "25.png" );
+    if( suit() == Spade   && card() == Eight ) return QLatin1String( "26.png" );
+    if( suit() == Heart   && card() == Eight ) return QLatin1String( "27.png" );
+    if( suit() == Diamond && card() == Eight ) return QLatin1String( "28.png" );
+    if( suit() == Club    && card() == Seven ) return QLatin1String( "29.png" );
+    if( suit() == Spade   && card() == Seven ) return QLatin1String( "30.png" );
+    if( suit() == Heart   && card() == Seven ) return QLatin1String( "31.png" );
+    if( suit() == Diamond && card() == Seven ) return QLatin1String( "32.png" );
+    if( suit() == Club    && card() == Six ) return QLatin1String( "33.png" );
+    if( suit() == Spade   && card() == Six ) return QLatin1String( "34.png" );
+    if( suit() == Heart   && card() == Six ) return QLatin1String( "35.png" );
+    if( suit() == Diamond && card() == Six ) return QLatin1String( "36.png" );
+    if( suit() == Club    && card() == Five ) return QLatin1String( "37.png" );
+    if( suit() == Spade   && card() == Five ) return QLatin1String( "38.png" );
+    if( suit() == Heart   && card() == Five ) return QLatin1String( "39.png" );
+    if( suit() == Diamond && card() == Five ) return QLatin1String( "40.png" );
+    if( suit() == Club    && card() == Four ) return QLatin1String( "41.png" );
+    if( suit() == Spade   && card() == Four ) return QLatin1String( "42.png" );
+    if( suit() == Heart   && card() == Four ) return QLatin1String( "43.png" );
+    if( suit() == Diamond && card() == Four ) return QLatin1String( "44.png" );
+    if( suit() == Club    && card() == Three ) return QLatin1String( "45.png" );
+    if( suit() == Spade   && card() == Three ) return QLatin1String( "46.png" );
+    if( suit() == Heart   && card() == Three ) return QLatin1String( "47.png" );
+    if( suit() == Diamond && card() == Three ) return QLatin1String( "48.png");
+    if( suit() == Club    && card() == Two ) return QLatin1String( "49.png" );
+    if( suit() == Spade   && card() == Two ) return QLatin1String( "50.png" );
+    if( suit() == Heart   && card() == Two ) return QLatin1String( "51.png" );
+    if( suit() == Diamond && card() == Two ) return QLatin1String( "52.png" );
+    return QLatin1String( "" );
 }
 
 QString KCardInfo::svgName() const
 {
     QString s;
     if( card() == KCardInfo::Ace )
-        s += "1_";
+        s += QLatin1String( "1_" );
     if( card() == KCardInfo::King )
-        s += "king_";
+        s += QLatin1String( "king_" );
     if( card() == KCardInfo::Queen )
-        s += "queen_";
+        s += QLatin1String( "queen_" );
     if( card() == KCardInfo::Jack )
-        s += "jack_";
+        s += QLatin1String( "jack_" );
     if( card() == KCardInfo::Ten )
-        s += "10_";
+        s += QLatin1String( "10_" );
     if( card() == KCardInfo::Nine )
-        s += "9_";
+        s += QLatin1String( "9_" );
     if( card() == KCardInfo::Eight )
-        s += "8_";
+        s += QLatin1String( "8_" );
     if( card() == KCardInfo::Seven )
-        s += "7_";
+        s += QLatin1String( "7_" );
     if( card() == KCardInfo::Six )
-        s += "6_";
+        s += QLatin1String( "6_" );
     if( card() == KCardInfo::Five )
-        s += "5_";
+        s += QLatin1String( "5_" );
     if( card() == KCardInfo::Four )
-        s += "4_";
+        s += QLatin1String( "4_" );
     if( card() == KCardInfo::Three )
-        s += "3_";
+        s += QLatin1String( "3_" );
     if( card() == KCardInfo::Two )
-        s += "2_";
+        s += QLatin1String( "2_" );
     if( suit() == KCardInfo::Club )
-        s += "club";
+        s += QLatin1String( "club" );
     if( suit() == KCardInfo::Spade )
-        s += "spade";
+        s += QLatin1String( "spade" );
     if( suit() == KCardInfo::Diamond )
-        s += "diamond";
+        s += QLatin1String( "diamond" );
     if( suit() == KCardInfo::Heart )
-        s += "heart";
+        s += QLatin1String( "heart" );
     return s;
 }
 
-QPixmap doRender( const QString& element, KSvgRenderer* r, const QSize& s )
+QPixmap doRender( const QString& element, QSvgRenderer* r, const QSize& s )
 {
-    QImage img = QImage( s, QImage::Format_ARGB32 );
-    img.fill( qRgba( 0, 0, 255, 0 ) );
-    QPainter p( &img );
-    if( r->elementExists( element ) )
-    {
-        r->render( &p, element );
-    }else
-    {
-        r->render( &p, "layer1" );
-    }
+    QPixmap pix = QPixmap( s );
+    pix.fill( Qt::transparent );
+    QPainter p( &pix );
+    r->render( &p, element );
     p.end();
-    return QPixmap::fromImage( img );
+    return pix;
 }
 
 QString keyForPixmap( const QString& theme, const QString& element, const QSize& s )
 {
-    return theme + '_' + element + '_'
-                  + QString::number( s.width() ) + '_'
+    return theme + QLatin1Char( '_' ) + element + QLatin1Char( '_' )
+                  + QString::number( s.width() ) + QLatin1Char( '_' )
                   + QString::number( s.height() );
 }
 
+QSvgRenderer* KCardCachePrivate::frontRenderer()
+{
+    if ( !frontSvgRenderer )
+    {
+        kDebug() << "Loading front SVG renderer";
+        frontSvgRenderer = new QSvgRenderer( CardDeckInfo::frontSVGFilePath( frontTheme ) );
+    }
+    return frontSvgRenderer;
+}
+
+QSvgRenderer* KCardCachePrivate::backRenderer()
+{
+    if ( !backSvgRenderer )
+    {
+        kDebug() << "Loading back SVG renderer";
+        backSvgRenderer = new QSvgRenderer( CardDeckInfo::backSVGFilePath( backTheme ) );
+    }
+    return backSvgRenderer;
+}
+
 void KCardCachePrivate::ensureNonNullPixmap( QPixmap& pix )
 {
     if( pix.isNull() )
@@ -269,18 +283,27 @@ void KCardCachePrivate::ensureNonNullPixmap( QPixmap& pix )
 
 QPixmap KCardCachePrivate::renderFrontSvg( const QString& element )
 {
+    kDebug() << "Renderering" << element << "in main thread.";
     QMutexLocker l( frontRendererMutex );
-    return doRender( element, frontRenderer, size );
+    return doRender( element, frontRenderer(), size );
 }
 
 QPixmap KCardCachePrivate::renderBackSvg( const QString& element )
 {
     QMutexLocker l( backRendererMutex );
-    return doRender( element, backRenderer, size );
+    return doRender( element, backRenderer(), size );
 }
 
-LoadThread::LoadThread( KCardCache::LoadInfos l_, KCardCachePrivate* d_ )
-    : d( d_ ), infos( l_ ), killMutex( new QMutex )
+void KCardCachePrivate::submitRendering( const QString& key, const QImage& image )
+{
+    kDebug() << "Received render of" << key << "from rendering thread.";
+    QPixmap pix = QPixmap::fromImage( image );
+    QMutexLocker l( frontcacheMutex );
+    frontcache->insert( key, pix );
+}
+
+LoadThread::LoadThread( KCardCachePrivate* d_ )
+    : d( d_ ), doKill( false ), killMutex( new QMutex )
 {
 }
 
@@ -299,6 +322,11 @@ void LoadThread::setBackTheme( const QString & backTheme_ )
     backTheme = backTheme_;
 }
 
+void LoadThread::setElementsToLoad( const QStringList& elements_ )
+{
+    elementsToRender = elements_;
+}
+
 void LoadThread::kill()
 {
     QMutexLocker l(killMutex);
@@ -308,71 +336,29 @@ void LoadThread::kill()
 void LoadThread::run()
 {
     {
-        QMutexLocker l( killMutex );
-        if( doKill )
-            return;
+        // Load the renderer even if we don't have any elements to render.
+        QMutexLocker l( d->frontRendererMutex );
+        d->frontRenderer();
     }
-    if( infos & KCardCache::LoadBackSide )
+
+    foreach( const QString& element, elementsToRender )
     {
-        bool found = false;
-        QString key = keyForPixmap( backTheme, "back", size );
-        QPixmap pix;
         {
-            QMutexLocker l( d->backcacheMutex );
-            if( d->backcache && d->backcache->find( key, pix ) )
-                found = true;
+            QMutexLocker l( killMutex );
+            if( doKill )
+                return;
         }
-        if( !found )
+        QImage img = QImage( size, QImage::Format_ARGB32 );
+        img.fill( Qt::transparent );
+        QPainter p( &img );
         {
-            {
-                QMutexLocker l( d->backRendererMutex );
-                pix = doRender( "back", d->backRenderer, size );
-            }
-            {
-                QMutexLocker l( d->backcacheMutex );
-                if( d->backcache )
-                    d->backcache->insert( keyForPixmap( backTheme, "back", size ), pix );
-            }
-        }
-    }
-    {
-        QMutexLocker l( killMutex );
-        if( doKill )
-            return;
-    }
-    if( infos & KCardCache::LoadFrontSide )
-    {
-        int numCards = 53;
-        if( infos & KCardCache::Load52Cards )
-            numCards = 52;
-        if( infos & KCardCache::Load32Cards )
-            numCards = 32;
-        for( int i = 0; i < numCards ; i++ )
-        {
-            QString element = fullDeckList[i].svgName();
-            QString key = keyForPixmap( frontTheme, element, size );
-            QPixmap pix;
-            {
-                QMutexLocker l( killMutex );
-                if( doKill )
-                    return;
-            }
-            {
-                QMutexLocker l( d->frontcacheMutex );
-                if( d->frontcache && d->frontcache->find( key, pix ) )
-                    continue;
-            }
-            {
-                QMutexLocker l( d->frontRendererMutex );
-                pix = doRender( element, d->frontRenderer, size );
-            }
-            {
-                QMutexLocker l( d->frontcacheMutex );
-                QPixmap tmp;
-                if( d->frontcache )
-                    d->frontcache->insert( key, pix );
-            }
+            QMutexLocker l( d->frontRendererMutex );
+            d->frontRenderer()->render( &p, element );
         }
+        p.end();
+
+        QString key = keyForPixmap( frontTheme, element, size );
+        emit renderingDone( key, img );
     }
 }
 
@@ -385,10 +371,9 @@ KCardCache::KCardCache()
     d->backcacheMutex = new QMutex();
     d->frontRendererMutex = new QMutex();
     d->backRendererMutex = new QMutex();
-    d->frontRenderer = 0;
-    d->backRenderer = 0;
+    d->frontSvgRenderer = 0;
+    d->backSvgRenderer = 0;
     d->loadThread = 0;
-    
 }
 
 KCardCache::~KCardCache()
@@ -405,8 +390,8 @@ KCardCache::~KCardCache()
     delete d->backcacheMutex;
     delete d->frontRendererMutex;
     delete d->backRendererMutex;
-    delete d->frontRenderer;
-    delete d->backRenderer;
+    delete d->frontSvgRenderer;
+    delete d->backSvgRenderer;
     delete d;
 }
 
@@ -415,7 +400,7 @@ QPixmap KCardCache::backside( int variant ) const
     QPixmap pix;
     if( d->backTheme.isEmpty() || d->size.isEmpty() )
         return pix;
-    QString element = "back";
+    QString element = QLatin1String( "back" );
     if( variant > 0 )
     {
         element += QString::number(variant);
@@ -456,7 +441,7 @@ QPixmap KCardCache::frontside( const KCardInfo& info ) const
     if( d->frontTheme.isEmpty() || d->size.isEmpty() )
         return pix;
     QString key = keyForPixmap( d->frontTheme, info.svgName() , d->size );
-    
+
     if( !CardDeckInfo::isSVGFront( d->frontTheme ) )
     {
         QMutexLocker l( d->frontcacheMutex );
@@ -465,7 +450,7 @@ QPixmap KCardCache::frontside( const KCardInfo& info ) const
             QMatrix matrix;
             QImage img;
             QString filename = CardDeckInfo::frontDir( d->frontTheme )
-                    + '/' + info.pngName();
+                    + QLatin1Char( '/' ) + info.pngName();
             bool ret = img.load( filename, "PNG" );
             if( !ret )
                 return QPixmap();
@@ -504,7 +489,7 @@ void KCardCache::setFrontTheme( const QString& theme )
     {
         QMutexLocker l( d->frontcacheMutex );
         delete d->frontcache;
-        d->frontcache = new KPixmapCache( QString( "kdegames-cards_%1" ).arg( theme ) );
+        d->frontcache = new KPixmapCache( QString::fromLatin1(  "kdegames-cards_%1" ).arg( theme ) );
         d->frontcache->setUseQPixmapCache( true );
         QDateTime dt;
         if( CardDeckInfo::isSVGFront( theme ) )
@@ -514,7 +499,7 @@ void KCardCache::setFrontTheme( const QString& theme )
         } else
         {
             QDir carddir( CardDeckInfo::frontDir( theme ) );
-            foreach( const QFileInfo& entry, carddir.entryInfoList( QStringList() << "*.png" ) )
+            foreach( const QFileInfo& entry, carddir.entryInfoList( QStringList() << QLatin1String( "*.png" ) ) )
             {
                 if( dt.isNull() || dt < entry.lastModified() )
                 {
@@ -530,10 +515,8 @@ void KCardCache::setFrontTheme( const QString& theme )
     }
     {
         QMutexLocker l( d->frontRendererMutex );
-        delete d->frontRenderer;
-        d->frontRenderer = 0;
-        if ( CardDeckInfo::isSVGFront( theme ) )
-            d->frontRenderer = new KSvgRenderer( CardDeckInfo::frontSVGFilePath( theme ) );
+        delete d->frontSvgRenderer;
+        d->frontSvgRenderer = 0;
     }
     d->frontTheme = theme;
 }
@@ -548,7 +531,7 @@ void KCardCache::setBackTheme( const QString& theme )
     {
         QMutexLocker l( d->backcacheMutex );
         delete d->backcache;
-        d->backcache = new KPixmapCache( QString( "kdegames-cards_%1" ).arg( theme ) );
+        d->backcache = new KPixmapCache( QString::fromLatin1(  "kdegames-cards_%1" ).arg( theme ) );
         d->backcache->setUseQPixmapCache( true );
         QDateTime dt;
         if( CardDeckInfo::isSVGBack( theme ) )
@@ -567,10 +550,8 @@ void KCardCache::setBackTheme( const QString& theme )
     }
     {
         QMutexLocker l( d->backRendererMutex );
-        delete d->backRenderer;
-        d->backRenderer = 0;
-        if ( CardDeckInfo::isSVGBack( theme ) )
-            d->backRenderer = new KSvgRenderer( CardDeckInfo::backSVGFilePath( theme ) );
+        delete d->backSvgRenderer;
+        d->backSvgRenderer = 0;
     }
     d->backTheme = theme;
 }
@@ -582,62 +563,116 @@ QString KCardCache::backTheme() const
 
 void KCardCache::loadTheme( LoadInfos infos )
 {
-    if( d->loadThread->isRunning() )
+    if( d->loadThread && d->loadThread->isRunning() )
     {
         d->loadThread->kill();
         d->loadThread->wait();
-        delete d->loadThread;
     }
-    d->loadThread = new LoadThread( infos, d );
+    delete d->loadThread;
+
+    // We have to compile the list of elements to load here, because we can't
+    // check the contents of the KPixmapCache from outside the GUI thread.
+    QStringList elements;
+    QPixmap pix;
+    if( infos & KCardCache::LoadFrontSide )
+    {
+        int numCards;
+        if( infos & KCardCache::Load53Cards )
+            numCards = 53;
+        else if( infos & KCardCache::Load52Cards )
+            numCards = 52;
+        else
+            numCards = 32;
+
+        for( int i = 0; i < numCards ; i++ )
+        {
+            QString element = fullDeckList[i].svgName();
+            QString key = keyForPixmap( d->frontTheme, element, d->size );
+            {
+                QMutexLocker l( d->frontcacheMutex );
+                if( d->frontcache && !d->frontcache->find( key, pix ) )
+                    elements << element;
+            }
+        }
+    }
+
+    d->loadThread = new LoadThread( d );
     d->loadThread->setBackTheme( d->backTheme );
     d->loadThread->setFrontTheme( d->frontTheme );
     d->loadThread->setSize( d->size );
-    d->loadThread->start();
+    d->loadThread->setElementsToLoad( elements );
+    d->connect( d->loadThread, SIGNAL(renderingDone(QString,QImage)), SLOT(submitRendering(QString,QImage)), Qt::QueuedConnection );
+    d->loadThread->start( QThread::IdlePriority );
 }
 
 QSizeF KCardCache::defaultFrontSize( const KCardInfo& info ) const
 {
-    QSizeF size;
     if( d->frontTheme.isEmpty() )
-        return size;
+        return QSizeF();
+
+    QPixmap pix;
+    QString key = d->frontTheme + QLatin1Char( '_' ) + info.svgName() + QLatin1String( "_default" );
+    {
+        QMutexLocker( d->frontcacheMutex );
+        if ( d->frontcache && d->frontcache->find( key, pix ) )
+            return pix.size();
+    }
+
     if( !CardDeckInfo::isSVGFront( d->frontTheme ) )
     {
-        QImage img;
-        if( img.load( CardDeckInfo::frontDir( d->frontTheme )
-            + '/' + info.pngName(), "PNG" ) )
-            size = img.size();
+        pix.load( CardDeckInfo::frontDir( d->frontTheme ) + QLatin1Char( '/' ) + info.pngName(), "PNG" );
     }else
     {
         QMutexLocker( d->frontRendererMutex );
-        size = d->frontRenderer->boundsOnElement( info.svgName() ).size();
+        QSizeF size = d->frontRenderer()->boundsOnElement( info.svgName() ).size();
+        pix = QPixmap( size.toSize() );
     }
-    return size;
+
+    {
+        QMutexLocker( d->frontcacheMutex );
+        if( d->frontcache )
+            d->frontcache->insert( key, pix );
+    }
+
+    return pix.size();
 }
 
 QSizeF KCardCache::defaultBackSize( int variant ) const
 {
-    QSizeF size;
-    if( d->backTheme.isEmpty() ) 
-        return size;
-    QString element = "back";
+    if( d->backTheme.isEmpty() )
+        return QSizeF();
+
+    QString element = QLatin1String( "back" );
     if( variant > 0 )
     {
         element += QString::number(variant);
     }
-    
+
+    QPixmap pix;
+    QString key = d->backTheme + QLatin1Char( '_' ) + element + QLatin1String( "_default" );
+    {
+        QMutexLocker( d->backcacheMutex );
+        if ( d->backcache && d->backcache->find( key, pix ) )
+            return pix.size();
+    }
+
     if( !CardDeckInfo::isSVGBack( d->backTheme ) )
     {
-        QImage img;
-        if( img.load( CardDeckInfo::backFilename( d->backTheme ), "PNG" ) )
-        {
-            size = img.size();
-        }
+        pix.load( CardDeckInfo::backFilename( d->backTheme ), "PNG" );
     }else
     {
         QMutexLocker( d->backRendererMutex );
-        size = d->backRenderer->boundsOnElement( element ).size();
+        QSizeF size = d->backRenderer()->boundsOnElement( element ).size();
+        pix = QPixmap( size.toSize() );
     }
-    return size;
+
+    {
+        QMutexLocker( d->backcacheMutex );
+        if( d->backcache )
+            d->frontcache->insert( key, pix );
+    }
+
+    return pix.size();
 }
 
 void KCardCache::invalidateFrontside()
