@@ -79,7 +79,7 @@ public:
 };
 
 // ------------------- GAME CLASS --------------------------
-KGame::KGame(int cookie,QObject* parent) 
+KGame::KGame(int cookie,QObject* parent)
     : KGameNetwork(cookie,parent),
       d( new KGamePrivate )
 {
@@ -137,16 +137,18 @@ bool KGame::reset()
 void KGame::deletePlayers()
 {
 // kDebug(11001) ;
- KGamePlayerList tmp = d->mPlayerList; // in case of PolicyClean player=d->mPlayerList.first() is infinite
- while (!tmp.isEmpty())
-         delete tmp.takeFirst();
+ qDeleteAll(d->mPlayerList);
+ //NOTE by majewsky: An earlier implementation copied the mPlayerList before
+ //deleting the elements with a takeFirst loop. I therefore chose not to clear()
+ //the list in order not to break anything. The old code had the following
+ //comment: "in case of PolicyClean player=d->mPlayerList.first() is infinite"
 // kDebug(11001) << "done";
 }
 
 void KGame::deleteInactivePlayers()
 {
- while (!d->mInactivePlayerList.isEmpty())
-         delete d->mInactivePlayerList.takeFirst();
+ qDeleteAll(d->mInactivePlayerList);
+ d->mInactivePlayerList.clear();
 }
 
 bool KGame::load(const QString& filename,bool reset)
@@ -220,13 +222,13 @@ bool KGame::loadgame(QDataStream &stream, bool network,bool resetgame)
  // If there is additional data to be loaded before players are loaded then do
  // this here.
  emit signalLoadPrePlayers(stream);
- 
+
  // Switch back on the direct emitting of signals and emit the
- // queued signals for properties. 
- // Unlocks properties before loading players in order to make game 
- // initializations related to properties before using them in players 
+ // queued signals for properties.
+ // Unlocks properties before loading players in order to make game
+ // initializations related to properties before using them in players
  // initialization
- dataHandler()->unlockDirectEmit(); 
+ dataHandler()->unlockDirectEmit();
 
  // Load Playerobjects
  uint playercount;
@@ -335,7 +337,7 @@ void KGame::savePlayers(QDataStream &stream, KGamePlayerList *list)
  qint32 cnt=list->count();
  kDebug(11001) << "Saving KGame" << cnt << "KPlayer objects";
  stream << cnt;
- 
+
  for ( KGamePlayerList::iterator it = playerList()->begin(); it!=playerList()->end();it++ )
  {
    savePlayer(stream,*it);
@@ -510,7 +512,7 @@ bool KGame::removePlayer(KPlayer * player, quint32 receiver)
 {//transmit to all clients, or to receiver only
  if (!player)
  {
-   kFatal(11001) << "trying to remove NULL player in KGame::removePlayer()";
+   kFatal(11001) << "trying to remove NULL player in KGame::removePlayer(  )" ;
    return false;
  }
  kDebug(11001) << ": id (" << player->id() << ") to be removed" << player;
@@ -1195,7 +1197,7 @@ void KGame::setupGameContinue(QDataStream& stream, quint32 sender)
 
   // Only to the client first , as the client will add players
   sendSystemMessage(sender, KGameMessage::IdGameSetupDone, sender);
-  
+
   //Finally delete content of the newPlayerList
   qDeleteAll(newPlayerList);
   newPlayerList.clear();
