@@ -1,6 +1,6 @@
 /*
     This file is part of the KDE games library
-    Copyright (C) 2001 Martin Heni (martin@heni-online.de)
+    Copyright (C) 2001 Martin Heni (kde at heni-online.de)
     Copyright (C) 2001 Andreas Beckermann (b_mann@gmx.de)
 
     This library is free software; you can redistribute it and/or
@@ -14,23 +14,33 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
-*/
-/*
-    $Id: kgameio.h 377450 2005-01-11 12:51:23Z mlaurent $
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
+
 #ifndef __KGAMEIO_H__
 #define __KGAMEIO_H__
 
-#include <qstring.h>
-#include <qobject.h>
-#include <kdemacros.h>
+#include <QtCore/QString>
+#include <QtCore/QObject>
+
+#include <libkdegames_export.h>
+
+class QEvent;
+class QGraphicsScene;
+class QKeyEvent;
+class QMouseEvent;
 class KPlayer;
 class KGame;
-class KProcess;
+class KGameIOPrivate;
+class KGameKeyIOPrivate;
+class KGameMouseIOPrivate;
+class KGameProcessIOPrivate;
+class KGameComputerIOPrivate;
 
 /**
+ *  \class KGameIO kgameio.h <KGame/KGameIO>
+ * 
  *  \short Base class for IO devices for games
  *
  *  This is the master class for
@@ -50,9 +60,9 @@ class KProcess;
  *  To achieve this you have to make all of your player inputs through a
  *  KGameIO. You will usually call KGameIO::sendInput to do so. 
  *
- *  @author Martin Heni <martin@heni-online.de>
+ *  @author Martin Heni <kde at heni-online.de>
  */
-class KDE_EXPORT KGameIO : public QObject
+class KDEGAMES_EXPORT KGameIO : public QObject
 {
   Q_OBJECT
 
@@ -86,7 +96,7 @@ public:
     *
     * @return the player this IO device is plugged into
     */
-   KPlayer *player() const {return mPlayer;}
+   KPlayer *player() const;
 
    /**
     * Equivalent to player()->game()
@@ -101,7 +111,7 @@ public:
     *
     * @param p the player
     */
-   void setPlayer(KPlayer *p) {mPlayer=p;}
+   void setPlayer(KPlayer *p);
 
    /**
     * Init this device by setting the player and e.g. sending an
@@ -128,9 +138,9 @@ public:
     /**
      * Send an input message using @ref KPlayer::forwardInput
      **/
-    bool sendInput(QDataStream& stream, bool transmit = true, Q_UINT32 sender = 0);
+    bool sendInput(QDataStream& stream, bool transmit = true, quint32 sender = 0);
 
-signals:
+Q_SIGNALS:
     /**
      * Signal generated when @ref KPlayer::myTurn changes. This can either be
      * when you get the turn status or when you lose it.
@@ -166,15 +176,20 @@ signals:
 
 
 private:
-   KPlayer *mPlayer;
+    friend class KGameIOPrivate;
+    KGameIOPrivate *const d;
+
+    Q_DISABLE_COPY(KGameIO)
 };
 
 /**
+ *  \class KGameKeyIO kgameio.h <KGame/KGameIO>
+ * 
  *  The KGameKeyIO class. It is used to process keyboard input
  *  from a widget and create moves for the player it belongs to.
- *  @author Martin Heni <martin@heni-online.de>
+ *  @author Martin Heni <kde at heni-online.de>
  */
-class KDE_EXPORT KGameKeyIO : public KGameIO
+class KDEGAMES_EXPORT KGameKeyIO : public KGameIO
 {
   Q_OBJECT
 
@@ -209,7 +224,7 @@ public:
      */
     virtual int rtti() const;
 
-signals:
+Q_SIGNALS:
       /**
        * Signal handler for keyboard events. This function is called
        * on every keyboard event. If appropriate it can generate a
@@ -221,7 +236,7 @@ signals:
        * Example:
        * \code
        * KPlayer *player=input->player(); // Get the player
-       * Q_INT32 key=e->key();
+       * qint32 key=e->key();
        * stream << key;
        * eatevent=true;
        * \endcode
@@ -238,14 +253,22 @@ protected:
        * Internal method to process the events
        */
        bool eventFilter( QObject *o, QEvent *e );
+
+private:
+    friend class KGameKeyIOPrivate;
+    KGameKeyIOPrivate *const d;
+
+    Q_DISABLE_COPY(KGameKeyIO)
 };
 
 /**
+ *  \class KGameMouseIO kgameio.h <KGame/KGameIO>
+ * 
  *  The KGameMouseIO class. It is used to process mouse input
  *  from a widget and create moves for the player it belongs to.
- *  @author Martin Heni <martin@heni-online.de>
+ *  @author Martin Heni <kde at heni-online.de>
  */
-class KDE_EXPORT KGameMouseIO : public KGameIO
+class KDEGAMES_EXPORT KGameMouseIO : public KGameIO
 {
   Q_OBJECT
     
@@ -265,7 +288,8 @@ public:
      * @param parent The widget whose events should be captured
      * @param trackmouse enables mouse tracking (gives mouse move events)
      */
-    KGameMouseIO(QWidget *parent,bool trackmouse=false);
+    explicit KGameMouseIO(QWidget *parent,bool trackmouse=false);
+    explicit KGameMouseIO(QGraphicsScene *parent,bool trackmouse=false);
     virtual ~KGameMouseIO();
     
     /**
@@ -281,7 +305,7 @@ public:
      */
     virtual int rtti() const; 
 
-signals:
+Q_SIGNALS:
       /**
        * Signal handler for mouse events. This function is called
        * on every mouse event. If appropriate it can generate a
@@ -291,7 +315,7 @@ signals:
        * Example:
        * \code
        * KPlayer *player=input->player(); // Get the player
-       * Q_INT32 button=e->button();
+       * qint32 button=e->button();
        * stream << button;
        * eatevent=true;
        * \endcode
@@ -309,18 +333,24 @@ protected:
       */
       bool eventFilter( QObject *o, QEvent *e );
 
+private:
+    friend class KGameMouseIOPrivate;
+    KGameMouseIOPrivate *const d;
+
+    Q_DISABLE_COPY(KGameMouseIO)
 };
 
 
 /**
+ *  \class KGameProcessIO kgameio.h <KGame/KGameIO>
+ * 
  *  The KGameProcessIO class. It is used to create a computer player
  *  via a separate process and communicate transparetly with it.
  *  Its counterpart is the @ref KGameProcess class which needs
  *  to be used by the computer player. See its documentation
  *  for the definition of the computer player.
- *  @author Martin Heni <martin@heni-online.de>
  */
-class KDE_EXPORT KGameProcessIO : public KGameIO
+class KDEGAMES_EXPORT KGameProcessIO : public KGameIO
 {
   Q_OBJECT
     
@@ -364,7 +394,7 @@ public:
      * @param receiver - not used
      * @param sender - who send the message
      */
-    void sendMessage(QDataStream &stream,int msgid, Q_UINT32 receiver, Q_UINT32 sender);
+    void sendMessage(QDataStream &stream,int msgid, quint32 receiver, quint32 sender);
 
     /**
      * Send a system message to the process. This is analogous to the sendMessage
@@ -376,7 +406,7 @@ public:
      * @param receiver - not used
      * @param sender - who send the message
      */
-    void sendSystemMessage(QDataStream &stream, int msgid, Q_UINT32 receiver, Q_UINT32 sender);
+    void sendSystemMessage(QDataStream &stream, int msgid, quint32 receiver, quint32 sender);
 
     /** 
      * Init this device by setting the player and e.g. sending an
@@ -403,16 +433,16 @@ public:
     /**
      * Internal ~ombined function for all message handling 
      **/
-    void sendAllMessages(QDataStream &stream,int msgid, Q_UINT32 receiver, Q_UINT32 sender, bool usermsg);
+    void sendAllMessages(QDataStream &stream,int msgid, quint32 receiver, quint32 sender, bool usermsg);
 
-  protected slots:
+  protected Q_SLOTS:
   /**
   * Internal message handler to receive data from the process
   */
     void receivedMessage(const QByteArray& receiveBuffer);
 
   
-signals:
+Q_SIGNALS:
   /**
    * A computer query message is received. This is a 'dummy'
    * message sent by the process if it needs to communicate
@@ -430,7 +460,7 @@ signals:
    *    if (no==1)     // but YOU have to do this in the process player
    *    {
    *      QByteArray buffer;
-   *      QDataStream out(buffer,IO_WriteOnly);
+   *      QDataStream out(buffer,QIODevice::WriteOnly);
    *      reply->sendSystemMessage(out,4242,0,0);  // lets reply something...
    *    }
    *  }
@@ -450,15 +480,21 @@ signals:
   */
   void signalIOAdded(KGameIO *game,QDataStream &stream,KPlayer *p,bool *send);
 
-
-protected:
+  /** Text is received by the process on STDERR. This is usually a debug string.
+    * @param msg The text
+    */
+  void signalReceivedStderr(QString msg);
 
 private:
-  class KGameProcessIOPrivate;
-  KGameProcessIOPrivate* d;
+    friend class KGameProcessIOPrivate;
+    KGameProcessIOPrivate *const d;
+
+    Q_DISABLE_COPY(KGameProcessIO)
 };
 
 /**
+ *  \class KGameComputerIO kgameio.h <KGame/KGameIO>
+ * 
  *  \brief KGameIO variant for real-time games
  *
  *  The KGameComputerIO class. It is used to create a LOCAL computer player
@@ -474,7 +510,7 @@ private:
  *
  *  @author  <b_mann@gmx.de>
  */
-class KDE_EXPORT KGameComputerIO : public KGameIO
+class KDEGAMES_EXPORT KGameComputerIO : public KGameIO
 {
   Q_OBJECT
     
@@ -523,7 +559,7 @@ public:
      **/
     void unpause();
     
-public slots:
+public Q_SLOTS:
     /**
      * Works kind of similar to QCanvas::advance. Increase the internal
      * advance counter. If @p reactionPeriod is reached the counter is set back to
@@ -541,7 +577,7 @@ public slots:
      **/
     virtual void advance();
   
-signals:
+Q_SIGNALS:
     /**
      * This signal is emitted when your computer player is meant to do
      * something, or better is meant to be allowed to do something.
@@ -555,11 +591,10 @@ protected:
     virtual void reaction();
 
 private:
-    void init();
- 
-private:
-    class KGameComputerIOPrivate;
-    KGameComputerIOPrivate* d;
+    friend class KGameComputerIOPrivate;
+    KGameComputerIOPrivate *const d;
+
+    Q_DISABLE_COPY(KGameComputerIO)
 };
 
 
