@@ -1,6 +1,6 @@
 /*
     This file is part of the KDE games library
-    Copyright (C) 2001 Martin Heni (martin@heni-online.de)
+    Copyright (C) 2001 Martin Heni (kde at heni-online.de)
     Copyright (C) 2001 Andreas Beckermann (b_mann@gmx.de)
 
     This library is free software; you can redistribute it and/or
@@ -14,18 +14,15 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
-*/
-/*
-    $Id: kplayer.cpp 208352 2003-02-21 15:11:01Z andreas $
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
-
+#include "kplayer.h"
 #include "kgame.h"
 #include "kgameio.h"
-#include "kplayer.h"
 #include "kgamemessage.h"
+#include "kgameproperty.h"
 #include "kgamepropertyhandler.h"
 
 #include <kdebug.h>
@@ -46,7 +43,16 @@ public:
       mNetworkPlayer = 0;
    }
 
-   Q_UINT32 mId;
+   KGame *mGame;
+   bool mActive;      // active player
+   KPlayer::KGameIOList mInputList;
+
+   // GameProperty
+   KGamePropertyBool mAsyncInput;  // async input allowed
+   KGamePropertyBool mMyTurn;      // Is it my turn to play (only useful if not async)?
+   KGamePropertyInt  mUserId;      // a user defined id
+
+   quint32 mId;
    bool mVirtual; // virtual player
    int mPriority; // tag for replacement
 
@@ -59,60 +65,51 @@ public:
    KGamePropertyQString mGroup;
 };
 
-KPlayer::KPlayer() : QObject(0,0)
+KPlayer::KPlayer()
+    : QObject(), d(new KPlayerPrivate)
 {
  init();
 }
 
-KPlayer::KPlayer(KGame* game) : QObject(0, 0)
-{
- init();
- game->addPlayer(this);
-}
-
 void KPlayer::init()
 {
 // note that NO KGame object exists here! so we cannot use KGameProperty::send!
-   kdDebug(11001) << k_funcinfo << ": this=" << this << ", sizeof(this)="<<sizeof(KPlayer) << endl;
-   kdDebug(11001) << "sizeof(m_Group)="<<sizeof(d->mGroup)<<endl;
-   d = new KPlayerPrivate;
+   kDebug(11001) << ": this=" << this << ", sizeof(this)="<<sizeof(KPlayer);
+   kDebug(11001) << "sizeof(m_Group)="<<sizeof(d->mGroup);
 
    d->mProperties.registerHandler(KGameMessage::IdPlayerProperty,
                                   this,SLOT(sendProperty(int, QDataStream&, bool*)),
                                        SLOT(emitSignal(KGamePropertyBase *)));
    d->mVirtual=false;
-   mActive=true;
-   mGame=0;
+   d->mActive=true;
+   d->mGame=0;
    d->mId=0; // "0" is always an invalid ID!
    d->mPriority=0;
    // I guess we cannot translate the group otherwise no
    // international conenctions are possible
 
-   mUserId.registerData(KGamePropertyBase::IdUserId, this, i18n("UserId"));
-   mUserId.setLocal(0);
+   d->mUserId.registerData(KGamePropertyBase::IdUserId, this, i18n("UserId"));
+   d->mUserId.setLocal(0);
    d->mGroup.registerData(KGamePropertyBase::IdGroup, this, i18n("Group"));
    d->mGroup.setLocal(i18n("default"));
    d->mName.registerData(KGamePropertyBase::IdName, this, i18n("Name"));
    d->mName.setLocal(i18n("default"));
 
-   mAsyncInput.registerData(KGamePropertyBase::IdAsyncInput, this, i18n("AsyncInput"));
-   mAsyncInput.setLocal(false);
-   mMyTurn.registerData(KGamePropertyBase::IdTurn, this, i18n("myTurn"));
-   mMyTurn.setLocal(false);
-   mMyTurn.setEmittingSignal(true);
-   mMyTurn.setOptimized(false);
+   d->mAsyncInput.registerData(KGamePropertyBase::IdAsyncInput, this, i18n("AsyncInput"));
+   d->mAsyncInput.setLocal(false);
+   d->mMyTurn.registerData(KGamePropertyBase::IdTurn, this, i18n("myTurn"));
+   d->mMyTurn.setLocal(false);
+   d->mMyTurn.setEmittingSignal(true);
+   d->mMyTurn.setOptimized(false);
 }
 
 KPlayer::~KPlayer()
 {
-  kdDebug(11001) << k_funcinfo << ": this=" << this <<", id=" << this->id() << endl;
+  kDebug(11001) << ": this=" << this <<", id=" << this->id();
 
   // Delete IODevices
-  KGameIO *input;
-  while((input=mInputList.first()))
-  {
-    delete input;
-  }
+  qDeleteAll(d->mInputList);
+  d->mInputList.clear();
   if (game())
   {
     game()->playerDeleted(this);
@@ -122,10 +119,65 @@ KPlayer::~KPlayer()
 // himself
   d->mProperties.clear();
   delete d;
-//  kdDebug(11001) << k_funcinfo << " done" << endl;
+  kDebug(11001) << "done";
 }
 
-bool KPlayer::forwardMessage(QDataStream &msg,int msgid,Q_UINT32 receiver,Q_UINT32 sender)
+int KPlayer::rtti() const
+{
+  return 0;
+}
+
+KPlayer::KGameIOList* KPlayer::ioList()
+{
+  return &d->mInputList;
+}
+
+void KPlayer::setGame(KGame *game)
+{
+  d->mGame = game;
+}
+
+KGame* KPlayer::game() const
+{
+  return d->mGame;
+}
+
+void KPlayer::setAsyncInput(bool a)
+{
+  d->mAsyncInput = a;
+}
+
+bool KPlayer::asyncInput() const
+{
+  return d->mAsyncInput.value();
+}
+
+bool KPlayer::isActive() const
+{
+  return d->mActive;
+}
+
+void KPlayer::setActive(bool v)
+{
+  d->mActive = v;
+}
+
+int KPlayer::userId() const
+{
+  return d->mUserId.value();
+}
+
+void KPlayer::setUserId(int i)
+{
+  d->mUserId = i;
+}
+
+bool KPlayer::myTurn() const
+{
+  return d->mMyTurn.value();
+}
+
+bool KPlayer::forwardMessage(QDataStream &msg,int msgid,quint32 receiver,quint32 sender)
 {
   if (!isActive())
   {
@@ -135,11 +187,11 @@ bool KPlayer::forwardMessage(QDataStream &msg,int msgid,Q_UINT32 receiver,Q_UINT
   {
     return false;
   }
-  kdDebug(11001) << k_funcinfo << ": to game sender="<<sender<<"" << "recv="<<receiver <<"msgid="<<msgid << endl;
+  kDebug(11001) << ": to game sender="<<sender<<"" << "recv="<<receiver <<"msgid="<<msgid;
   return game()->sendSystemMessage(msg,msgid,receiver,sender);
 }
 
-bool KPlayer::forwardInput(QDataStream &msg,bool transmit,Q_UINT32 sender)
+bool KPlayer::forwardInput(QDataStream &msg,bool transmit,quint32 sender)
 {
   if (!isActive())
   {
@@ -150,10 +202,10 @@ bool KPlayer::forwardInput(QDataStream &msg,bool transmit,Q_UINT32 sender)
     return false;
   }
 
-  kdDebug(11001) << k_funcinfo << ": to game playerInput(sender="<<sender<<")" << endl;
+  kDebug(11001) << ": to game playerInput(sender="<<sender<<")";
   if (!asyncInput() && !myTurn())
   {
-    kdDebug(11001) << k_funcinfo << ": rejected cause it is not our turn" << endl;
+    kDebug(11001) << ": rejected cause it is not our turn";
     return false;
   }
 
@@ -163,17 +215,17 @@ bool KPlayer::forwardInput(QDataStream &msg,bool transmit,Q_UINT32 sender)
   // where it really looks at the input
   if (transmit)
   {
-    kdDebug(11001) << "indirect playerInput" << endl;
+    kDebug(11001) << "indirect playerInput";
     return game()->sendPlayerInput(msg,this,sender);
   }
   else
   {
-    kdDebug(11001) << "direct playerInput" << endl;
+    kDebug(11001) << "direct playerInput";
     return game()->systemPlayerInput(msg,this,sender);
   }
 }
 
-void KPlayer::setId(Q_UINT32 newid)
+void KPlayer::setId(quint32 newid)
 {
   // Needs to be after the sendProcess
   d->mId=newid;
@@ -192,7 +244,7 @@ void KPlayer::setName(const QString& name)
 const QString& KPlayer::name() const
 { return d->mName.value(); }
 
-Q_UINT32 KPlayer::id() const
+quint32 KPlayer::id() const
 { return d->mId; }
 
 KGamePropertyHandler * KPlayer::dataHandler()
@@ -222,7 +274,7 @@ bool KPlayer::addGameIO(KGameIO *input)
   {
     return false;
   }
-  mInputList.append(input); 
+  d->mInputList.append(input);
   input->initIO(this); // set player and init device
   return true;
 }
@@ -230,19 +282,20 @@ bool KPlayer::addGameIO(KGameIO *input)
 // input=0, remove all
 bool KPlayer::removeGameIO(KGameIO *targetinput,bool deleteit)
 {
-  kdDebug(11001) << k_funcinfo << ": " << targetinput << " delete=" << deleteit<< endl;
+  kDebug(11001) << ":" << targetinput << "delete=" << deleteit;
   bool result=true;
   if (!targetinput) // delete all
   {
     KGameIO *input;
-    while((input=mInputList.first()))
+    while(!d->mInputList.isEmpty())
     {
+      input = d->mInputList.first();
       if (input) removeGameIO(input,deleteit);
     }
   }
   else
   {
-//    kdDebug(11001) << "remove IO " << targetinput << endl;
+//    kDebug(11001) << "remove IO" << targetinput;
     if (deleteit)
     {
       delete targetinput;
@@ -250,22 +303,27 @@ bool KPlayer::removeGameIO(KGameIO *targetinput,bool deleteit)
     else
     {
       targetinput->setPlayer(0);
-      result=mInputList.remove(targetinput);
+      result = d->mInputList.removeAll(targetinput);
     }
   }
   return result;
 }
 
+bool KPlayer::hasRtti(int rtti) const
+{
+  return findRttiIO(rtti) != 0;
+}
+
 KGameIO * KPlayer::findRttiIO(int rtti) const
 {
-  QPtrListIterator<KGameIO> it(mInputList);
-  while (it.current())
+  QListIterator<KGameIO*> it(d->mInputList);
+  while (it.hasNext())
   {
-    if (it.current()->rtti() == rtti)
+    KGameIO* curGameIO = it.next();
+    if (curGameIO->rtti() == rtti)
     {
-      return it.current();
+      return curGameIO;
     }
-    ++it;
   }
   return 0;
 }
@@ -273,18 +331,17 @@ KGameIO * KPlayer::findRttiIO(int rtti) const
 int KPlayer::calcIOValue()
 {
   int value=0;
-  QPtrListIterator<KGameIO> it(mInputList);
-  while (it.current())
+  QListIterator<KGameIO*> it(d->mInputList);
+  while (it.hasNext())
   {
-    value|=it.current()->rtti();
-    ++it;
+    value|=it.next()->rtti();
   }
   return value;
 }
 
 bool KPlayer::setTurn(bool b, bool exclusive)
 {
-  kdDebug(11001) << k_funcinfo << ": " << id() << " (" << this << ") to " << b << endl;
+  kDebug(11001) << ":" << id() << " (" << this << ") to" << b;
   if (!isActive())
   {
     return false;
@@ -293,27 +350,26 @@ bool KPlayer::setTurn(bool b, bool exclusive)
   // if we get to do an exclusive turn all other players are disallowed
   if (exclusive && b && game())
   {
-     KPlayer *player;
-     KGame::KGamePlayerList *list=game()->playerList();
-     for ( player=list->first(); player != 0; player=list->next() )
+     for ( KGame::KGamePlayerList::iterator it = game()->playerList()->begin();
+            it!=game()->playerList()->end(); ++it)
      {
-       if (player==this)
+       if ((*it)==this)
        {
          continue;
        }
-       player->setTurn(false,false);
+       (*it)->setTurn(false,false);
      }
   }
 
   // Return if nothing changed
-  mMyTurn = b;
+  d->mMyTurn = b;
 
   return true;
 }
 
 bool KPlayer::load(QDataStream &stream)
 {
-  Q_INT32 id,priority;
+  qint32 id,priority;
   stream >> id >> priority;
   setId(id);
   setNetworkPriority(priority);
@@ -322,15 +378,15 @@ bool KPlayer::load(QDataStream &stream)
   //FIXME: maybe set all properties setEmitSignal(false) before?
   d->mProperties.load(stream);
 
-  Q_INT16 cookie;
+  qint16 cookie;
   stream >> cookie;
   if (cookie==KPLAYER_LOAD_COOKIE)
   {
-      kdDebug(11001) << "   Player loaded propertly"<<endl;
+      kDebug(11001) << "   Player loaded propertly";
   }
   else
   {
-      kdError(11001) << "   Player loading error. probably format error"<<endl;
+      kError(11001) << "   Player loading error. probably format error";
   }
 
   // emit signalLoad(stream);
@@ -339,20 +395,20 @@ bool KPlayer::load(QDataStream &stream)
 
 bool KPlayer::save(QDataStream &stream)
 {
-  stream << (Q_INT32)id() << (Q_INT32)networkPriority();
+  stream << (qint32)id() << (qint32)networkPriority();
 
   d->mProperties.save(stream);
 
-  stream << (Q_INT16)KPLAYER_LOAD_COOKIE;
+  stream << (qint16)KPLAYER_LOAD_COOKIE;
 
   //emit signalSave(stream);
   return true;
 }
 
 
-void KPlayer::networkTransmission(QDataStream &stream,int msgid,Q_UINT32 sender)
+void KPlayer::networkTransmission(QDataStream &stream,int msgid,quint32 sender)
 {
-  //kdDebug(11001) << k_funcinfo ": msgid=" << msgid << " sender=" << sender << " we are=" << id() << endl;
+  //kDebug(11001) ": msgid=" << msgid << "sender=" << sender << "we are=" << id();
   // PlayerProperties processed
   bool issender;
   if (game())
@@ -371,16 +427,16 @@ void KPlayer::networkTransmission(QDataStream &stream,int msgid,Q_UINT32 sender)
   {
     case KGameMessage::IdPlayerInput:
       {
-        kdDebug(11001) << k_funcinfo << ": Got player move "
-	        << "KPlayer (virtual) forwards it to the game object" << endl;
+        kDebug(11001) << ": Got player move "
+	        << "KPlayer (virtual) forwards it to the game object";
         forwardInput(stream,false);
       }
     break;
     default:
         emit signalNetworkData(msgid - KGameMessage::IdUser,
 	        ((QBuffer*)stream.device())->readAll(),sender,this);
-        kdDebug(11001) << k_funcinfo << ": "
-	        << "User data msgid " << msgid << endl;
+        kDebug(11001) << ": "
+	        << "User data msgid" << msgid;
     break;
   }
 
@@ -413,12 +469,11 @@ void KPlayer::emitSignal(KGamePropertyBase *me)
   // Notify KGameIO (Process) for a new turn
   if (me->id()==KGamePropertyBase::IdTurn)
   {
-    //kdDebug(11001) << k_funcinfo << ": for KGamePropertyBase::IdTurn " << endl;
-    QPtrListIterator<KGameIO> it(mInputList);
-    while (it.current())
+    //kDebug(11001) << ": for KGamePropertyBase::IdTurn";
+    QListIterator<KGameIO*> it(d->mInputList);
+    while (it.hasNext())
     {
-      it.current()->notifyTurn(mMyTurn.value());
-      ++it;
+      it.next()->notifyTurn(d->mMyTurn.value());
     }
   }
   emit signalPropertyChanged(me,this);
@@ -427,20 +482,20 @@ void KPlayer::emitSignal(KGamePropertyBase *me)
 // --------------------- DEBUG --------------------
 void KPlayer::Debug()
 {
-   kdDebug(11001) << "------------------- KPLAYER -----------------------" << endl;
-   kdDebug(11001) << "this:    " << this << endl;
-   kdDebug(11001) << "rtti:    " << rtti() << endl;
-   kdDebug(11001) << "id  :    " << id() << endl;
-   kdDebug(11001) << "Name :   " << name() << endl;
-   kdDebug(11001) << "Group:   " << group() << endl;
-   kdDebug(11001) << "Async:   " << asyncInput() << endl;
-   kdDebug(11001) << "myTurn:  " << myTurn() << endl;
-   kdDebug(11001) << "Virtual: " << isVirtual() << endl;
-   kdDebug(11001) << "Active:  " << isActive() << endl;
-   kdDebug(11001) << "Priority:" << networkPriority() << endl;
-   kdDebug(11001) << "Game   : " << game() << endl;
-   kdDebug(11001) << "#IOs:    " << mInputList.count() << endl;
-   kdDebug(11001) << "---------------------------------------------------" << endl;
+   kDebug(11001) << "------------------- KPLAYER -----------------------";
+   kDebug(11001) << "this:    " << this;
+   kDebug(11001) << "rtti:    " << rtti();
+   kDebug(11001) << "id  :    " << id();
+   kDebug(11001) << "Name :   " << name();
+   kDebug(11001) << "Group:   " << group();
+   kDebug(11001) << "Async:   " << asyncInput();
+   kDebug(11001) << "myTurn:  " << myTurn();
+   kDebug(11001) << "Virtual:" << isVirtual();
+   kDebug(11001) << "Active:  " << isActive();
+   kDebug(11001) << "Priority:" << networkPriority();
+   kDebug(11001) << "Game   :" << game();
+   kDebug(11001) << "#IOs:    " << d->mInputList.count();
+   kDebug(11001) << "---------------------------------------------------";
 }
 
 #include "kplayer.moc"
