@@ -92,12 +92,12 @@ Mainwindow::Mainwindow(QWidget* parent)
   #ifndef NDEBUG
   #ifdef SRC_DIR
   kDebug() << "Found SRC_DIR =" << SRC_DIR;
-  KGlobal::dirs()->addResourceDir("lskattheme",QString(SRC_DIR)+QString("/grafix/"));
+  KGlobal::dirs()->addResourceDir("lskattheme",QLatin1String( SRC_DIR)+QString("/grafix/"));
   #endif
   #endif
 
   // Read theme files
-  QStringList themeList =  KGlobal::dirs()->findAllResources("lskattheme", "*.desktop", KStandardDirs::NoDuplicates);
+  QStringList themeList =  KGlobal::dirs()->findAllResources("lskattheme", QLatin1String( "*.desktop" ), KStandardDirs::NoDuplicates);
   if (themeList.isEmpty())
   {
     KMessageBox::error(this, i18n("Installation error: No theme list found."));
@@ -135,7 +135,7 @@ Mainwindow::Mainwindow(QWidget* parent)
 
   // TODO: Bugfix: Needs to be here if initGUI is befure readProperties
   if (global_debug>0) kDebug() << "Setting current theme item to" << mThemeIndexNo;
-  ((KSelectAction*)ACTION("theme"))->setCurrentItem(mThemeIndexNo);
+  ((KSelectAction*)ACTION(QLatin1String( "theme" )))->setCurrentItem(mThemeIndexNo);
 
 
   // Get the card deck
@@ -329,7 +329,7 @@ AbstractInput* Mainwindow::createInput(
 void Mainwindow::startGame()
 {
   // Enable game action
-  QString endName = KStandardGameAction::name(KStandardGameAction::End);
+  QLatin1String endName( KStandardGameAction::name(KStandardGameAction::End) );
   ACTION(endName)->setEnabled(true);
 
   // Deal cards to player - Shuffle card deck and reset pile
@@ -367,7 +367,7 @@ void Mainwindow::startGame()
 // Here a game over is signalled
 void Mainwindow::gameOver(int /*winner*/)
 {
-  QString endName = KStandardGameAction::name(KStandardGameAction::End);
+  QLatin1String endName( KStandardGameAction::name(KStandardGameAction::End) );
   ACTION(endName)->setEnabled(false);
   statusBar()->showMessage(i18n("Game Over. Please start a new game."));
 
@@ -414,7 +414,7 @@ void Mainwindow::initGUI()
 
   // Determine start player
   KSelectAction* startPlayerAct = new KSelectAction(i18n("Starting Player"), this);
-  actionCollection()->addAction("startplayer", startPlayerAct);
+  actionCollection()->addAction( QLatin1String( "startplayer" ), startPlayerAct);
   connect(startPlayerAct, SIGNAL(triggered(int)), this, SLOT(menuStartplayer()));
   startPlayerAct->setToolTip(i18n("Changing starting player..."));
   startPlayerAct->setWhatsThis(i18n("Chooses which player begins the next game."));
@@ -428,7 +428,7 @@ void Mainwindow::initGUI()
 
   // Determine who player player 1
   KSelectAction* player1Act = new KSelectAction(i18n("Player &1 Played By"), this);
-  actionCollection()->addAction("player1", player1Act);
+  actionCollection()->addAction( QLatin1String( "player1" ), player1Act);
   connect(player1Act, SIGNAL(triggered(int)), this, SLOT(menuPlayer1By()));
   player1Act->setToolTip(i18n("Changing who plays player 1..."));
   player1Act->setWhatsThis(i18n("Changing who plays player 1."));
@@ -440,7 +440,7 @@ void Mainwindow::initGUI()
 
   // Determine who player player 2
   KSelectAction* player2Act = new KSelectAction(i18n("Player &2 Played By"), this);
-  actionCollection()->addAction("player2", player2Act);
+  actionCollection()->addAction( QLatin1String( "player2" ), player2Act);
   connect(player2Act, SIGNAL(triggered(int)), this, SLOT(menuPlayer2By()));
   player2Act->setToolTip(i18n("Changing who plays player 2..."));
   player2Act->setWhatsThis(i18n("Changing who plays player 2."));
@@ -452,7 +452,7 @@ void Mainwindow::initGUI()
   themes.sort();
 
   KSelectAction* themeAct = new KSelectAction(i18n("&Theme"), this);
-  actionCollection()->addAction("theme", themeAct);
+  actionCollection()->addAction( QLatin1String( "theme" ), themeAct);
   themeAct->setItems(themes);
   connect( themeAct, SIGNAL(triggered(int)), SLOT(changeTheme(int)) );
   if (global_debug>0) kDebug() << "Setting current theme item to" << mThemeIndexNo;
@@ -461,7 +461,7 @@ void Mainwindow::initGUI()
   themeAct->setWhatsThis(i18n("Changing theme."));
 
   // Choose card deck
-  KAction *action1 = actionCollection()->addAction("select_carddeck");
+  KAction *action1 = actionCollection()->addAction( QLatin1String( "select_carddeck" ));
   action1->setText(i18n("Select &Card Deck..."));
   action1->setShortcuts( KShortcut( Qt::Key_F10 ) );
   connect(action1, SIGNAL(triggered(bool)), this, SLOT(menuCardDeck()));
@@ -469,7 +469,7 @@ void Mainwindow::initGUI()
   action1->setWhatsThis(i18n("Choose how the cards should look."));
 
   // Change player names
-  action = actionCollection()->addAction("change_names");
+  action = actionCollection()->addAction( QLatin1String( "change_names" ));
   action->setText(i18n("&Change Player Names..."));
   connect(action, SIGNAL(triggered(bool)), this, SLOT(menuPlayerNames()));
   if (global_demo_mode) action->setEnabled(false);
@@ -479,7 +479,7 @@ void Mainwindow::initGUI()
 // Choose start player
 void Mainwindow::menuStartplayer()
 {
-  int i=((KSelectAction *)ACTION("startplayer"))->currentItem();
+  int i=((KSelectAction *)ACTION(QLatin1String( "startplayer" )))->currentItem();
   setStartPlayer(i);
 }
 
@@ -498,7 +498,7 @@ void Mainwindow::changeTheme(int idx)
 // Select input for player 1
 void Mainwindow::menuPlayer1By()
 {
-  int i = ((KSelectAction *)ACTION("player1"))->currentItem();
+  int i = ((KSelectAction *)ACTION(QLatin1String( "player1" )))->currentItem();
   mLSkatConfig->setInputType(0, (InputDeviceType)i);
 }
 
@@ -506,7 +506,7 @@ void Mainwindow::menuPlayer1By()
 // Select input for player 2
 void Mainwindow::menuPlayer2By()
 {
-  int i = ((KSelectAction *)ACTION("player2"))->currentItem();
+  int i = ((KSelectAction *)ACTION(QLatin1String( "player2" )))->currentItem();
   mLSkatConfig->setInputType(1, (InputDeviceType)i);
 }
 
@@ -660,7 +660,7 @@ void Mainwindow::menuPlayerNames()
 void Mainwindow::setStartPlayer(int no)
 {
   mStartPlayer = no;
-  ((KSelectAction *)ACTION("startplayer"))->setCurrentItem(mStartPlayer);
+  ((KSelectAction *)ACTION(QLatin1String( "startplayer" )))->setCurrentItem(mStartPlayer);
 }
 
 
@@ -671,12 +671,12 @@ void Mainwindow::setInputType(int no, InputDeviceType type)
   // Player 1
   if (no == 0)
   {
-    ((KSelectAction *)ACTION("player1"))->setCurrentItem((int)type);
+    ((KSelectAction *)ACTION(QLatin1String( "player1" )))->setCurrentItem((int)type);
     p = mLSkatConfig->player(0);
   }
   else if (no == 1)
   {
-    ((KSelectAction *)ACTION("player2"))->setCurrentItem((int)type);
+    ((KSelectAction *)ACTION(QLatin1String( "player2" )))->setCurrentItem((int)type);
     p = mLSkatConfig->player(1);
   }
 
