@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
+ *   Copyright 2010 Johannes Löhnert <loehnert.kde@gmx.de>
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public
@@ -19,37 +20,70 @@
 #include "piece.h"
 #include "piece_p.h"
 #include "scene.h"
+#include "settings.h"
 
 #include <QApplication>
 #include <QGraphicsSceneMouseEvent>
 #include <QPalette>
 #include <QPropertyAnimation>
 
+void Palapeli::Piece::commonInit(const Palapeli::PieceVisuals& pieceVisuals)
+{
+	Palapeli::SelectionAwarePixmapItem* pieceItem = new Palapeli::SelectionAwarePixmapItem(pieceVisuals.pixmap(), this);
+	connect(pieceItem, SIGNAL(selectedChanged(bool)), SLOT(pieceItemSelectedChanged(bool)));
+	m_pieceItem = pieceItem;
+	m_pieceItem->setOffset(pieceVisuals.offset());
+	//initialize behavior
+	m_pieceItem->setAcceptedMouseButtons(Qt::LeftButton);
+	m_pieceItem->setCursor(Qt::OpenHandCursor);
+	m_pieceItem->setFlag(QGraphicsItem::ItemIsSelectable);
+	// replacing m_pieceItems pixmap (in rerenderBevel()) causes weird pixel errors
+	// when using fast transformation. SmoothTransformation looks better anyway...
+	m_pieceItem->setTransformationMode(Qt::SmoothTransformation);
+}
+
+Palapeli::Piece::Piece(const QImage& pieceImage, const QPoint& offset)
+	: m_pieceItem(0)
+	, m_inactiveShadowItem(0)
+	, m_activeShadowItem(0)
+	, m_animator(0)
+{
+	//create bevel map if wanted
+	if (Settings::pieceBevelsEnabled())
+	{
+		//NOTE: The bevel map calculation and application is divided because we
+		//need different result visuals for different turning angles when piece
+		//rotation gets implementation.
+		const QSize size = pieceImage.size();
+		const int radius = 0.04 * (size.width() + size.height());
+		const Palapeli::BevelMap bevelMap = Palapeli::calculateBevelMap(pieceImage, radius);
+		commonInit(Palapeli::PieceVisuals(Palapeli::applyBevelMap(pieceImage, bevelMap, /* angle= */ 0), offset));
+	}
+	else
+		commonInit(Palapeli::PieceVisuals(pieceImage, offset));
+}
+
 Palapeli::Piece::Piece(const Palapeli::PieceVisuals& pieceVisuals, const Palapeli::PieceVisuals& shadowVisuals)
 	: m_pieceItem(0)
 	, m_inactiveShadowItem(0)
 	, m_activeShadowItem(0)
 	, m_animator(0)
 {
-	//initialize appearance
-	Palapeli::SelectionAwarePixmapItem* pieceItem = new Palapeli::SelectionAwarePixmapItem(pieceVisuals.pixmap, this);
-	connect(pieceItem, SIGNAL(selectedChanged(bool)), this, SLOT(pieceItemSelectedChanged(bool)));
-	m_pieceItem = pieceItem;
-	m_pieceItem->setOffset(pieceVisuals.offset);
+	commonInit(pieceVisuals);
 	if (!shadowVisuals.isNull())
 		createShadowItems(shadowVisuals);
-	//initialize behavior
-	m_pieceItem->setAcceptedMouseButtons(Qt::LeftButton);
-	m_pieceItem->setCursor(Qt::OpenHandCursor);
-	m_pieceItem->setFlag(QGraphicsItem::ItemIsSelectable);
 }
 
 //BEGIN visuals
 
-void Palapeli::Piece::createShadow()
+bool Palapeli::Piece::completeVisuals()
 {
-	if (!m_inactiveShadowItem)
+	if (Settings::pieceShadowsEnabled() && !m_inactiveShadowItem)
+	{
 		createShadowItems(Palapeli::createShadow(pieceVisuals(), m_atomicSize));
+		return true;
+	}
+	return false;
 }
 
 bool Palapeli::Piece::hasShadow() const
@@ -62,12 +96,12 @@ void Palapeli::Piece::createShadowItems(const Palapeli::PieceVisuals& shadowVisu
 	const QColor activeShadowColor = QApplication::palette().color(QPalette::Highlight);
 	const Palapeli::PieceVisuals activeShadowVisuals = Palapeli::changeShadowColor(shadowVisuals, activeShadowColor);
 	//create inactive shadow item
-	m_inactiveShadowItem = new QGraphicsPixmapItem(shadowVisuals.pixmap, this);
-	m_inactiveShadowItem->setOffset(shadowVisuals.offset);
+	m_inactiveShadowItem = new QGraphicsPixmapItem(shadowVisuals.pixmap(), this);
+	m_inactiveShadowItem->setOffset(shadowVisuals.offset());
 	m_inactiveShadowItem->setZValue(-2);
 	//create active shadow item and animator for its opacity
-	m_activeShadowItem = new QGraphicsPixmapItem(activeShadowVisuals.pixmap, this);
-	m_activeShadowItem->setOffset(activeShadowVisuals.offset);
+	m_activeShadowItem = new QGraphicsPixmapItem(activeShadowVisuals.pixmap(), this);
+	m_activeShadowItem->setOffset(activeShadowVisuals.offset());
 	m_activeShadowItem->setZValue(-1);
 	m_activeShadowItem->setOpacity(isSelected() ? 1.0 : 0.0);
 	m_animator = new QPropertyAnimation(this, "activeShadowOpacity", this);
@@ -85,16 +119,14 @@ QRectF Palapeli::Piece::sceneBareBoundingRect() const
 
 Palapeli::PieceVisuals Palapeli::Piece::pieceVisuals() const
 {
-	Palapeli::PieceVisuals result = { m_pieceItem->pixmap(), m_pieceItem->offset().toPoint() };
-	return result;
+	return Palapeli::PieceVisuals(m_pieceItem->pixmap(), m_pieceItem->offset().toPoint());
 }
 
 Palapeli::PieceVisuals Palapeli::Piece::shadowVisuals() const
 {
 	if (!m_inactiveShadowItem)
 		return Palapeli::PieceVisuals();
-	Palapeli::PieceVisuals result = { m_inactiveShadowItem->pixmap(), m_inactiveShadowItem->offset().toPoint() };
-	return result;
+	return Palapeli::PieceVisuals(m_inactiveShadowItem->pixmap(), m_inactiveShadowItem->offset().toPoint());
 }
 
 qreal Palapeli::Piece::activeShadowOpacity() const
@@ -113,7 +145,7 @@ void Palapeli::Piece::pieceItemSelectedChanged(bool selected)
 	//change visibility of active shadow
 	const qreal targetOpacity = selected ? 1.0 : 0.0;
 	const qreal opacityDiff = qAbs(targetOpacity - activeShadowOpacity());
-	if (opacityDiff != 0)
+	if (m_animator && opacityDiff != 0)
 	{
 		m_animator->setDuration(150 * opacityDiff);
 		m_animator->setStartValue(activeShadowOpacity());
